#define DRIVER_NAME			"adc" 

#define ADC_MAJOR_NUM		251
#define ADC_MINOR_NUM		0

/*Size allocated for 5 Control Register and 1 Data Channel  Register  */
#define ADC_REG_SIZE		128 
#define ADC_BASE_OFFSET	0xB0200C80


#define ADC_CHANNEL0_NUM							0x00
#define ADC_CHANNEL12_NUM							0x0B
#define ADC_INTERNAL_REF_VOLTAGE_IN_MVOLTS		1220
#define ADC_EXTERNAL_REF_VOLTAGE_IN_MVOLTS		2500//2.5v ref voltage for NetApp
#define ADC_NUM_BITS_FOR_RESOLUTION				10

/*ADC Control Registers*/
#define ADCC ((unsigned long *)(((unsigned long)adc_remap_addr)+0x00)) 
#define ADCPC  ((unsigned long *)(((unsigned long)adc_remap_addr)+0x04))
#define ADCCS ((unsigned long *)(((unsigned long)adc_remap_addr)+0x08))
#define ADCD ((unsigned long *)(((unsigned long)adc_remap_addr)+0x0C))
#define ADCIS ((unsigned long *)(((unsigned long)adc_remap_addr)+0x10))
#define ADCCD(x)  ((unsigned long *)(((unsigned long)adc_remap_addr)+0x40+((x)*4)))


#define SET_ADC_BIT(register,bit)							\
do{														\
	volatile uint32_t reg_buff=0;                       				\
	reg_buff=adc_read_reg((uint32_t)register); 				\
	reg_buff=reg_buff |(uint32_t)(1<<bit);					\
	adc_write_reg((uint32_t)register,reg_buff);				\
}while(0)


#define CLEAR_ADC_BIT(register,bit)						\
do{														\
	volatile uint32_t reg_buff=0;                       				\
	reg_buff=adc_read_reg((uint32_t)register); 				\
	reg_buff=reg_buff & (uint32_t)(~(1<<bit));				\
	adc_write_reg((uint32_t)register,reg_buff);				\
}while(0)

#if 0

/*ADC Control Registers*/
#define ADC_CONTROL_REG					0x80200C80   //ADCC
#define ADC_POWER_CONTROL_REG			0x80200C84   //ADCPC
#define ADC_CONVERSION_STATUS_REG		0x80200C88   //ADCCS
#define ADC_DATA_REG						0x80200C8c   //ADCD
#define ADC_INTERRUPT_STATUS_REG		0x80200C90   //ADCIS

/*ADC Channel Data Registers*/

#define ADC_CHANNEL_0_DATA_REG		      0x80200CC0    //ADCCD(x)
#define ADC_CHANNEL_1_DATA_REG		      0x80200CC4
#define ADC_CHANNEL_2_DATA_REG		      0x80200CC8
#define ADC_CHANNEL_3_DATA_REG		      0x80200CCC
#define ADC_CHANNEL_4_DATA_REG		      0x80200CD0
#define ADC_CHANNEL_5_DATA_REG		      0x80200CD4
#define ADC_CHANNEL_6_DATA_REG		      0x80200CD8
#define ADC_CHANNEL_7_DATA_REG		      0x80200CDC
#define ADC_CHANNEL_8_DATA_REG		      0x80200CE0
#define ADC_CHANNEL_9_DATA_REG		      0x80200CE4
#define ADC_CHANNEL_10_DATA_REG		      0x80200CE8
#define ADC_CHANNEL_11_DATA_REG		      0x80200CEC

#endif 


/* Ioctl  Function prototypes */
extern int adc_read_channel(void *adc_base_addr, uint16_t*adc_value, int channel);
extern int adc_get_resolution(void *adc_base_addr, uint16_t *adc_resolution);
extern int adc_get_reference_voltage(void *adc_base_addr, uint16_t *adc_ref_volatge);
extern int enable_external_reference_volt(void *adc_base_addr);
extern int disable_external_reference_volt(void *adc_base_addr);

void init_adc(void *adc_base_addr);
void exit_adc(void *adc_base_addr);


/*Register read and write functions */
void adc_write_reg(unsigned long addr, unsigned long val);
unsigned long adc_read_reg(unsigned long addr );















