/*
 *	crypt.h
 *
 *	DES and crypt algorithms.
 *
 *	$Id: //depot/prod/ontap/main/nctest/tests/services/security/authentication/tools/authtest/crypt.h#1 $
 */

/*
 * Copyright (c) 1989, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Tom Truscott.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.

     The setkey and encrypt entries  provide  (rather  primitive)
     access  to  the  DES algorithm.  The argument of setkey is a
     character array of length 64 containing only the  characters
     with  numerical  value  0  and 1.  If this string is divided
     into groups of  8,  the  low-order  bit  in  each  group  is
     ignored;  this  gives  a  56-bit  key  which is set into the
     machine.  This is the key that will be used with  the  above
     mentioned  algorithm  to encrypt or decrypt the string block
     with the function encrypt.

     The argument to the encrypt entry is a  character  array  of
     length  64  containing  only  the  characters with numerical
     value 0 and 1.  The argument array is modified in place to a
     similar  array  representing  the bits of the argument after
     having been subjected to the DES algorithm using the key set
     by  setkey. If edflag is zero, the argument is encrypted; if
     non-zero, it is decrypted.


 */

#ifndef __crypt_h__
#define __crypt_h__

/* If this is set to 'unsigned int' on a DEC Alpha, this gives about a
 * %20 speed up (longs are 8 bytes, int's are 4). */
#ifndef DES_LONG
#define DES_LONG unsigned long
#endif

typedef unsigned char des_cblock[8];
typedef struct des_ks_struct
	{
	union	{
		des_cblock _;
		/* make sure things are correct size on machines with
		 * 8 byte longs */
		DES_LONG pad[2];
		} ks;
#undef _
#define _	ks._
	} des_key_schedule[16];

/* refer crypt.c for comments */
typedef union {
	unsigned char b[8];
	struct {
#if defined(LONG_IS_32_BITS)
		/* long is often faster than a 32-bit bit field */
		long	i0;
		long	i1;
#else
		long	i0: 32;
		long	i1: 32;
#endif
	} b32;
} C_block;

#define _PASSWORD_EFMT1         '_'     /* extended encryption format */

#define NUM_ITERATIONS 725

/*
 * Encrypt (or decrypt if num_iter < 0) the 8 chars at "in" with abs(num_iter)
 * iterations of DES, using the the given 24-bit salt and the pre-computed key
 * schedule, and store the resulting 8 chars at "out" (in == out is permitted).
 *
 * NOTE: the performance of this routine is critically dependent on your
 * compiler and machine architecture.
 */
int des_cipher(const char* in,char * out,long salt,	int num_iter);
int des_cipher_r(const char* in,char * out,long salt,int num_iter,C_block *ks);

/*
 * Set up the key schedule from the key.
 * This takes the 64 bit key described above as 8 bytes
 */
int des_setkey(const char* key);
int des_setkey_r(const char* key,C_block *ks);

/*
#ifndef SunOS
int setkey(const char *key);
#endif
*/

/* This is the UNIX password encryption algo*/
char *crypt(const char *key, const char *setting);

#define OLDCRYPTBUFSZ	1+4+4+11+1	/* buffer size taken from crypt.c */
/* CSMP: Was old_crypt - caller provides a buffer for reentrancy. */ 
char *old_crypt_r(const char *key, const char *setting,
		  char buf[OLDCRYPTBUFSZ]);

#define KS_SIZE	16

#endif
