/*-------------------------------------------------------------------------
	Copyright (C) 2008-2009 American Megatrends Inc.
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-------------------------------------------------------------------------*/
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/wait.h>
#include <asm/io.h>
#include <asm/irq.h>
#include "i2c-data.h"
#include "i2c-hardware.h"

/* Enable this only for serious debugging */
//#define DEBUG

/* Prototypes */
irqreturn_t i2c_process_handler(int bus);

irqreturn_t i2c_handler(int this_irq, void *dev_id)
{
	volatile UINT32 ior;
	irqreturn_t vsc_irq = IRQ_NONE;

	ior = (i2c_vsc_read_reg((UINT32)INT_OBSERVE_REG));

	/* Check if the interrupt is generated by I2C0 */
	if (ior & I2C_OBSERVE_INT_0) {
		if(IRQ_HANDLED == i2c_process_handler(0)) {
			vsc_irq = IRQ_HANDLED;
		}
	}
	
	/* Check if the interrupt is generated by I2C1 */
	else if (ior & I2C_OBSERVE_INT_1) {   
		if (IRQ_HANDLED == i2c_process_handler(1)) {
	    vsc_irq = IRQ_HANDLED;
		}
	}

	/* Check if the interrupt is generated by I2C2 */
	else if (ior & I2C_OBSERVE_INT_2) {
		if (IRQ_HANDLED == i2c_process_handler(2)) {
			   vsc_irq = IRQ_HANDLED;
		}
	}
	
	/* Check if the interrupt is generated by I2C3 */
	else if (ior & I2C_OBSERVE_INT_3) {
		if (IRQ_HANDLED == i2c_process_handler(3)) {
			vsc_irq = IRQ_HANDLED;
		}
	}
	
	/* Check if the interrupt is generated by I2C4 */
	else if (ior & I2C_OBSERVE_INT_4) {
		if (IRQ_HANDLED == i2c_process_handler(4)) {
			vsc_irq = IRQ_HANDLED;
		}
	}
	
	/* Check if the interrupt is generated by I2C5 */
	else if (ior & I2C_OBSERVE_INT_5) {
		if (IRQ_HANDLED == i2c_process_handler(5)) {
			vsc_irq = IRQ_HANDLED;
		}
	}
    
	return vsc_irq;
}

irqreturn_t i2c_process_handler(int bus)
{
	UINT32 volatile reg_read;
	UINT32 volatile status_reg_read;

	status_reg_read = i2c_vsc_read_reg((UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_BIT_STATUS);
	reg_read = i2c_vsc_read_reg((UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_CNTRL);

#ifdef DEBUG
	printk(KERN_ERR "I2C%d:%02x:%02x:%02x:%02x\n", bus, (u8)status_reg_read, (u8)reg_read, (u8)vsc_data_ptr[bus].slave_addr, (u8)vsc_data_ptr[bus].state);
#endif
	/* Check for STOP detect flag */
	if (status_reg_read & (UINT32)I2C_STOP_DETECT_INT_STATUS) {

		if ((vsc_data_ptr[bus].state == STATE_MASTER_XMIT) || 
			(vsc_data_ptr[bus].state == STATE_MASTER_XMIT_RECV) ||
			(vsc_data_ptr[bus].state == STATE_MASTER_RECV)) {
#ifdef DEBUG
			printk(KERN_ERR "\nI2C%d Bus error detected under stop interrupt\n", bus);
#endif
			/* Issue soft reset */
			i2c_vsc_reset(bus);
	    
			/* Send stop condition - make sure we don't leave bus in busy state */
			i2c_vsc_write_reg((UINT32)(I2C_CLEAR_ALL_STATUS 
						| I2C_ENABLE_INTERFACE
						| I2C_ENABLE_INTERRUPT
						| I2C_SEND_STOP
						| I2C_ACKNOWLEDGEMENT),
						(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_CNTRL);

			/* Notify waiting thread (if one) of bus error */
			vsc_data_ptr[bus].op_status = OP_STATUS_BUS_ERR;
			vsc_data_ptr[bus].state = STATE_IDLE;
			wake_up(&vsc_data_ptr[bus].vsc_wait);

			return IRQ_HANDLED;

		} else if (vsc_data_ptr[bus].state == STATE_IDLE) {

			/* STOP detected when driver is in idle mode */


		} else if (vsc_data_ptr[bus].state == STATE_SLAVE) {

			/* Slave mode currently not implemented */
		}

		/* Finally reinitialize I2C internal data and hardware */
		i2c_init_internal_data(bus);
		i2c_init_hardware(bus);

		return IRQ_HANDLED;
	}
    
	/* If interrupt pending */
	if (status_reg_read & (UINT32)I2C_SERIAL_INTERFACE_INT_STATUS) {   

		/* If bus error */
		if (reg_read & (UINT32)I2C_BUS_ERROR) {
#ifdef DEBUG
			printk(KERN_ERR "\nI2C%d: Bus error!\n", bus);
#endif
    
			if ((vsc_data_ptr[bus].state == STATE_MASTER_XMIT) || 
				(vsc_data_ptr[bus].state == STATE_MASTER_XMIT_RECV) ||
				(vsc_data_ptr[bus].state == STATE_MASTER_RECV)) {

				/* Send stop condition */
				i2c_vsc_write_reg((UINT32)(I2C_CLEAR_ALL_STATUS 
							| I2C_ENABLE_INTERFACE
							| I2C_ENABLE_INTERRUPT
							| I2C_SEND_STOP
							| I2C_ACKNOWLEDGEMENT),
							(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_CNTRL);
			}
#if 1
			/* Reinitialize I2C internal data and hardware */
			i2c_init_internal_data(bus);
			i2c_init_hardware(bus);

			/* Issue soft reset */
			i2c_vsc_reset(bus);
#endif
			/* Notify waiting thread (if one) of bus error */
			vsc_data_ptr[bus].op_status = OP_STATUS_BUS_ERR;
			vsc_data_ptr[bus].state = STATE_IDLE;
			wake_up(&vsc_data_ptr[bus].vsc_wait);
			
			return IRQ_HANDLED;

		/* If we are in process of writing data to a slave device (master transmit) */
		} else if ((vsc_data_ptr[bus].state == STATE_MASTER_XMIT) || 
				(vsc_data_ptr[bus].state == STATE_MASTER_XMIT_RECV)) {

			/* If this is the first byte to be transferred */
			if (vsc_data_ptr[bus].TX_index == 0) {

				/* If failed to generate start condition */
				if (reg_read & I2C_START_STATUS_NOT) {
#ifdef DEBUG
					printk(KERN_ERR "\nI2C%d: Start generation failed on write!\n", bus);
#endif
					/* Issue soft reset */
					i2c_vsc_write_reg((UINT32)(I2C_SERIAL_INTERFACE_SOFT_RESET 
								| I2C_SERIAL_SCL_HIGH 
								| I2C_SERIAL_SDA_HIGH), 
								(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_BIT_CNTRL);

					/* Notify waiting thread (if one) of error (we use arbitration lost here) */
					vsc_data_ptr[bus].op_status = OP_STATUS_ARB_LOST;
					vsc_data_ptr[bus].state = STATE_IDLE;
					wake_up(&vsc_data_ptr[bus].vsc_wait);
					
					return IRQ_HANDLED;
				}
			}

			/* If arbitration lost */
			if (reg_read & I2C_LOST_ARBITRATION) {
#ifdef DEBUG
				printk(KERN_ERR "\nI2C%d: Arbitration lost on write\n", bus);
#endif
				/* Issue soft reset */
				i2c_vsc_write_reg((UINT32)(I2C_SERIAL_INTERFACE_SOFT_RESET 
							| I2C_SERIAL_SCL_HIGH 
							| I2C_SERIAL_SDA_HIGH), 
							(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_BIT_CNTRL);

				/* Notify waiting thread (if one) of arbitration lost */
				vsc_data_ptr[bus].op_status = OP_STATUS_ARB_LOST;
				vsc_data_ptr[bus].state = STATE_IDLE;
				wake_up(&vsc_data_ptr[bus].vsc_wait);
					
				return IRQ_HANDLED;
			}

			/* If NACK from slave */
			if (reg_read & I2C_LAST_RECEIVED_BIT) {
#ifdef DEBUG
				printk(KERN_ERR "I2C%d: Nack detected during write. Sending stop.\n", bus);
#endif               
				/* Send stop condition */
				i2c_vsc_write_reg((UINT32)(I2C_CLEAR_ALL_STATUS 
							| I2C_ENABLE_INTERFACE
							| I2C_ENABLE_INTERRUPT
							| I2C_SEND_STOP
							| I2C_ACKNOWLEDGEMENT),
							(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_CNTRL);

				/* Notify waiting thread (if one) of NACK */
				vsc_data_ptr[bus].op_status = OP_STATUS_NACK;
				vsc_data_ptr[bus].state = STATE_IDLE;
				wake_up(&vsc_data_ptr[bus].vsc_wait);
				
				return IRQ_HANDLED;
			}

			/* If this is just a scan */
			if (vsc_data_ptr[bus].scan) {
#ifdef DEBUG
				printk(KERN_ERR "I2C%d: Device scan detected during write.  Sending stop.\n", bus); 
#endif               
				/* Send stop condition */
				i2c_vsc_write_reg((UINT32)(I2C_CLEAR_ALL_STATUS 
							| I2C_ENABLE_INTERFACE
							| I2C_ENABLE_INTERRUPT
							| I2C_SEND_STOP
							| I2C_ACKNOWLEDGEMENT),
							(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_CNTRL);

				/* Notify waiting thread (if one) that device was found */
				vsc_data_ptr[bus].op_status = OP_STATUS_DEV_FOUND;
				vsc_data_ptr[bus].state = STATE_IDLE;
				wake_up(&vsc_data_ptr[bus].vsc_wait);

				return IRQ_HANDLED;
			}

			/* If we still have data to send */
			if (vsc_data_ptr[bus].TX_index < vsc_data_ptr[bus].TX_len) {
#ifdef DEBUG
				printk(KERN_ERR "I2C%d: Wrote byte:%02x\n", bus, 
					vsc_data_ptr[bus].TX_data[vsc_data_ptr[bus].TX_index]);
#endif               
				/* Sending buffered data onto the I2C bus */
				i2c_vsc_write_reg((UINT32)(vsc_data_ptr[bus].TX_data[vsc_data_ptr[bus].TX_index]),
							(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_DATA);

				vsc_data_ptr[bus].TX_index++;
			} else {

				if (vsc_data_ptr[bus].state == STATE_MASTER_XMIT_RECV) {
#ifdef DEBUG
					printk(KERN_ERR "I2C%d: Repeated start for 0x%02x\n", bus, vsc_data_ptr[bus].slave_addr << 1);
#endif               
					/* Set state to master recv before issuing repeated start */
					vsc_data_ptr[bus].state = STATE_MASTER_RECV;
					vsc_data_ptr[bus].TX_len = 0;
					vsc_data_ptr[bus].MasterRX_index = 0;

					/* Perform repeated start condition */
					i2c_vsc_write_reg((UINT32)(I2C_CLEAR_ALL_STATUS 
							| I2C_ENABLE_INTERFACE
							| I2C_ENABLE_INTERRUPT
							| I2C_SEND_START
							| I2C_ACKNOWLEDGEMENT),
							(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_CNTRL);

					/* Setup read operation slave address */
					i2c_vsc_write_reg((vsc_data_ptr[bus].slave_addr << 1) | READ_OPERATION_BIT, 
								(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_DATA);
				
				} else {
#ifdef DEBUG
					printk(KERN_ERR "I2C%d: Sending stop after write\n", bus);
#endif               
					/* Send stop condition */
					i2c_vsc_write_reg((UINT32)(I2C_CLEAR_ALL_STATUS 
								| I2C_ENABLE_INTERFACE
								| I2C_ENABLE_INTERRUPT
								| I2C_SEND_STOP
								| I2C_ACKNOWLEDGEMENT),
								(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_CNTRL);


					/* Notify waiting thread (if one) transmission is complete */
					vsc_data_ptr[bus].op_status = OP_STATUS_SUCCESS;
					vsc_data_ptr[bus].state = STATE_IDLE;
					wake_up(&vsc_data_ptr[bus].vsc_wait);
				}

			}

		/* If we are in process of reading data from a slave device (master receive) */
		} else if (vsc_data_ptr[bus].state == STATE_MASTER_RECV) {

			/* If this is the first byte to be read */
			if (vsc_data_ptr[bus].MasterRX_index == 0) {

				/* If failed to generate start condition */
				if (reg_read & I2C_START_STATUS_NOT) {
#ifdef DEBUG
					printk(KERN_ERR "\nI2C%d: Start generation failed on read!\n", bus);
#endif
					/* Issue soft reset */
					i2c_vsc_write_reg((UINT32)(I2C_SERIAL_INTERFACE_SOFT_RESET 
								| I2C_SERIAL_SCL_HIGH 
								| I2C_SERIAL_SDA_HIGH), 
								(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_BIT_CNTRL);

					/* Notify waiting thread (if one) of error (we use arbitration lost here) */
					vsc_data_ptr[bus].op_status = OP_STATUS_ARB_LOST;
					vsc_data_ptr[bus].state = STATE_IDLE;
					wake_up(&vsc_data_ptr[bus].vsc_wait);
										
					return IRQ_HANDLED;
				}
			}

			/* If arbitration lost */
			if (reg_read & I2C_LOST_ARBITRATION) {
#ifdef DEBUG
				printk(KERN_ERR "\nI2C%d: Arbitration lost on read\n", bus);
#endif
				/* Issue soft reset */
				i2c_vsc_write_reg((UINT32)(I2C_SERIAL_INTERFACE_SOFT_RESET 
							| I2C_SERIAL_SCL_HIGH 
							| I2C_SERIAL_SDA_HIGH), 
							(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_BIT_CNTRL);

				/* Notify waiting thread (if one) of arbitration lost */
				vsc_data_ptr[bus].op_status = OP_STATUS_ARB_LOST;
				vsc_data_ptr[bus].state = STATE_IDLE;					
				wake_up(&vsc_data_ptr[bus].vsc_wait);

				return IRQ_HANDLED;
			}

			/* If this is the last byte to read */
			if (vsc_data_ptr[bus].MasterRX_index == vsc_data_ptr[bus].MasterRX_len) {

				vsc_data_ptr[bus].state = STATE_IDLE;					

				/* Send stop condition */
				i2c_vsc_write_reg((UINT32)(I2C_CLEAR_ALL_STATUS 
							| I2C_ENABLE_INTERFACE
							| I2C_ENABLE_INTERRUPT
							| I2C_SEND_STOP
							| I2C_ACKNOWLEDGEMENT),
							(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_CNTRL);

				/* Read last byte */
				vsc_data_ptr[bus].MasterRX_data[vsc_data_ptr[bus].MasterRX_index - 1] = 
					i2c_vsc_read_reg((UINT32)i2c_vsc_io_base(bus)+VSC_I2CX_DATA);
#ifdef DEBUG
				printk(KERN_ERR "I2C%d: Read last byte:%02x\n", bus, 
					vsc_data_ptr[bus].MasterRX_data[vsc_data_ptr[bus].MasterRX_index - 1]);

				printk(KERN_ERR "I2C%d: Sending stop after read\n", bus);
#endif
				/* Notify waiting thread (if one) transmission is complete */
				vsc_data_ptr[bus].op_status = OP_STATUS_SUCCESS;
				wake_up(&vsc_data_ptr[bus].vsc_wait);

				return IRQ_HANDLED;
			}


			/* If NACK from slave */
			if (reg_read & (UINT32)I2C_LAST_RECEIVED_BIT) {
#ifdef DEBUG
				printk(KERN_ERR "I2C%d: Nack detected during read. Sending stop.\n", bus);
#endif
				/* Send stop condition */
				i2c_vsc_write_reg((UINT32)(I2C_CLEAR_ALL_STATUS 
							| I2C_ENABLE_INTERFACE
							| I2C_ENABLE_INTERRUPT
							| I2C_SEND_STOP
							| I2C_ACKNOWLEDGEMENT),
							(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_CNTRL);

				/* Notify waiting thread (if one) of NACK */
				vsc_data_ptr[bus].op_status = OP_STATUS_NACK;
				vsc_data_ptr[bus].state = STATE_IDLE;					
				wake_up(&vsc_data_ptr[bus].vsc_wait);

				return IRQ_HANDLED;
			}

			/* If we still have data to read */
			if (vsc_data_ptr[bus].MasterRX_index < vsc_data_ptr[bus].MasterRX_len) {

				/* Must do a dummy read for first byte */
				if (vsc_data_ptr[bus].MasterRX_index == 0) {
					/* Do a dummy read */
					u8 dummy_read;
					dummy_read = i2c_vsc_read_reg((UINT32)i2c_vsc_io_base(bus)+VSC_I2CX_DATA);
#ifdef DEBUG
					printk(KERN_ERR "I2C%d: Dummy read:%02x\n", bus, dummy_read);
#endif
				} else {

					/* Read byte from slave */
					vsc_data_ptr[bus].MasterRX_data[vsc_data_ptr[bus].MasterRX_index - 1] = 
						i2c_vsc_read_reg((UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_DATA);
#ifdef DEBUG
					printk(KERN_ERR "I2C%d: Read byte:%02x\n", bus, 
						vsc_data_ptr[bus].MasterRX_data[vsc_data_ptr[bus].MasterRX_index - 1]);
#endif
				}
				/* If we are about to read last byte (during next interrupt)*/
				if (vsc_data_ptr[bus].MasterRX_len - vsc_data_ptr[bus].MasterRX_index == 1) {

					/* Clear acknowledge bit */
					i2c_vsc_write_reg((UINT32)(I2C_ENABLE_INTERFACE
								| I2C_ENABLE_INTERRUPT),
								(UINT32)i2c_vsc_io_base(bus) + VSC_I2CX_CNTRL);
#ifdef DEBUG
					printk(KERN_ERR "I2C%d: Nack next byte we read\n", bus);
#endif
				}

				vsc_data_ptr[bus].MasterRX_index++;

			} else {
#ifdef DEBUG
				printk(KERN_ERR "I2C%d: Unexpected interrupt during master read\n", bus);
#endif
			}
		}

	/* Unkown interrupt condition */
	} else {
#ifdef DEBUG
		printk(KERN_ERR "I2C%d: Unexpected interrupt!\n", bus);
#endif
		/* Finally reinitialize I2C internal data and hardware */
		i2c_init_internal_data(bus);
		i2c_init_hardware(bus);
	
//LC		/* Disable Interrupt (will be re-enabled next time we attempt to transmit) */
//		i2c_vsc_disable_interrupt(bus, I2C_DISABLE_INTERRUPT);
	}

	return IRQ_HANDLED;
}

