
#include<linux/init.h>
#include<linux/module.h>
#include <linux/kernel.h>
#include <linux/poll.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/wait.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/cacheflush.h>
#include <asm/ioctl.h>

#include "jtag_vsc.h"


/*****Prototye Functions*************************/
static  int  jtag_vsc_init (void );
static void  jtag_vsc_exit (void);
static  int  jtag_vsc_open (struct inode *, struct file *);
static  int  jtag_vsc_release (struct inode *, struct file *);
static  int  jtag_vsc_read (struct file *, char *, size_t, loff_t *);
static  int  jtag_vsc_write (struct file *, const char *, size_t, loff_t *);
static  int  jtag_vsc_ioctl (struct inode *, struct file *, uint, ulong);



static struct file_operations jtag_vsc_ops;
static jtag_dev_t  jtag_dev;

int __init  
jtag_vsc_init(void)
{
	int  err, ret=0;
	uint32_t jtag_base;  
	jtag_dev_t  * dev= &jtag_dev; 
	uint32_t temp_32;

	
	printk(" VSC452 JTAG Init \n ");

	memset(&jtag_vsc_ops,0,sizeof(jtag_vsc_ops));
	jtag_vsc_ops.ioctl   = jtag_vsc_ioctl;
	jtag_vsc_ops.open    = jtag_vsc_open;
	jtag_vsc_ops.release = jtag_vsc_release;
	jtag_vsc_ops.read    = jtag_vsc_read;
	jtag_vsc_ops.write   = jtag_vsc_write;
	jtag_vsc_ops.owner   = THIS_MODULE;

	/*********Register Character Device ********************/

	if ((err = register_chrdev(JTAG_MAJOR_NO, JTAG_NAME, &jtag_vsc_ops)) < 0)
	{
		printk ("failed to register device (err %d)\n", err);
		ret = -ENODEV;
		goto fail;
	}  else {
		printk ("registered device %s\n", JTAG_NAME);
	}

	/* Remap registers */
	jtag_base = JTAG_BASE_OFFSET;
	if ((dev->jtag_addr = ioremap_nocache(jtag_base, JTAG_SIZE)) == NULL) 
	{
		printk("failed to map JTAG IO space %08x-%08x to memory",jtag_base, jtag_base + JTAG_SIZE - 1);
		ret = -EIO;
		goto fail;
	}

	/* check jtag memory region */
	if ((err = check_mem_region(jtag_base, JTAG_SIZE)) < 0)  
	{
		printk("JTAG IO space %08x-%08x already in use (err %d)",jtag_base, jtag_base + JTAG_SIZE - 1, err);
		ret = -EBUSY;
		goto fail;
	}

	/* request for the memory region */	
	if (request_mem_region(jtag_base, JTAG_SIZE, JTAG_NAME)  == NULL) 
	{ 
		printk("IO space %08x-%08x jtag request mem region failed (err %d)",jtag_base, jtag_base + JTAG_SIZE - 1, err);
		ret = -ENODEV;
		goto fail;

	}

	/* enable JTAG */
	jtag_write_reg((unsigned long)dev->jtag_addr, (1<<JTAG_JME));
	temp_32 = jtag_read_reg((unsigned long)dev->jtag_addr);
	jtag_write_reg((unsigned long)dev->jtag_addr, temp_32 | (1<<JTAG_TDOOE));

	printk("JTAG Module was sucessully loaded \n");
	return ret;

fail:
	jtag_vsc_exit();
	return ret;
  
}

	
static void  
jtag_vsc_exit(void)
{
	int err=0;
	jtag_dev_t  * dev= &jtag_dev; 
	uint32_t jtag_base;

	printk(" VSC452 JTAG Exit \n ");
	jtag_base = JTAG_BASE_OFFSET;

	/* release the memory occupied */
	release_mem_region(jtag_base, JTAG_SIZE); 

	/* unregister the device */
	unregister_chrdev(JTAG_MAJOR_NO, JTAG_NAME);

	/* Character Device unmapping   */ 
	if (dev->jtag_addr) 
	{
		iounmap(dev->jtag_addr); 
		dev->jtag_addr = NULL;
	}

	printk ( "JTAG Driver is unloaded sucessfully.\n");
	return;
	
 
}

/*iowrite ****/
inline void jtag_write_reg(unsigned long addr, unsigned long val)
{
        *((volatile uint32_t*)addr) = val;
}

/*********ioread*************/
inline unsigned long jtag_read_reg(unsigned long addr )
{
        return ((*(volatile uint32_t*)addr));
}


static  int
jtag_vsc_open (struct inode *inode, struct file *file)
{
      	return 0;
}

static  int 
jtag_vsc_release (struct inode *inode,struct file *file)
{
	return 0;
}

static  int  
jtag_vsc_read (struct file *file, char *buffer,size_t size, loff_t *offset)
{
	return 0;
}

static  int  
jtag_vsc_write (struct file *file, const char *buffer, size_t size, loff_t * offset)
{
	return 0;
}

static int  
jtag_vsc_ioctl (struct inode *inode, struct file *file, uint cmd , ulong arg)
{
	jtag_dev_t *dev;
	jtag_data_t jtag_data;
	jtag_tdi_t jtag_tdi;

	//printk("START :jtag_vsc_ioctl  \n");
	dev= &jtag_dev; 

      /*Getting data from user space to driver space  */

	switch(cmd)
	{
		  /* JTAG Functions - */ 
		case  SET_JTAG :
			if ( copy_from_user(&jtag_data, (void*)arg,sizeof(jtag_data_t)) )
			return -EFAULT;
			if (0!=vsc_setjtag(jtag_data.jtag_pin, jtag_data.jtag_level))
			return -1;
 		 	break;
		
		case  GET_TDI :
			
			if ( copy_from_user(&jtag_tdi, (void*)arg,sizeof(jtag_tdi_t)) )
			return -EFAULT;
			jtag_tdi.data=vsc_getjtag();
                   if (copy_to_user((void*)arg, &jtag_tdi , sizeof(jtag_tdi_t))) return -EFAULT;
			break;
			
			
		 default :
			
			return -EINVAL;
		}
		return 0;
}




module_init(jtag_vsc_init);
module_exit(jtag_vsc_exit);


int   
vsc_setjtag( unsigned char jtag_pin, unsigned char jtag_level )
{
	jtag_dev_t  * dev= &jtag_dev; 
	uint32_t temp_32;

	temp_32=jtag_read_reg((unsigned long)dev->jtag_addr);

	switch (jtag_pin) {
		case JTAG_TMS:
			if (jtag_level) {
				temp_32 |= (1<<JTAG_TMSOV);
			} else {
				temp_32 &= ~(1<<JTAG_TMSOV);
			}
			break;
		case JTAG_TCK:
			if (jtag_level) {
				temp_32 |= (1<<JTAG_TCKOV);
			} else {
				temp_32 &= ~(1<<JTAG_TCKOV);
			}
			break;
		case JTAG_TDO:
			if (jtag_level) {
				temp_32 |= (1<<JTAG_TDOOV);
			} else {
				temp_32 &= ~(1<<JTAG_TDOOV) ;
			}

			break;
		default:
			break;
	}
	jtag_write_reg((unsigned long)dev->jtag_addr, temp_32);
	return 0;
}

unsigned char   
vsc_getjtag( )
{
	jtag_dev_t  * dev= &jtag_dev; 
	volatile uint32_t Value;
	unsigned char bit;
	Value=jtag_read_reg((unsigned long)dev->jtag_addr);
	bit = (Value >> JTAG_TDIRV) & 0x01;

	return bit;
}


MODULE_DESCRIPTION("VSC452 JTAG Driver");
MODULE_AUTHOR("NetApp Inc");
MODULE_LICENSE("GPL");

	


