#ifndef                     _VSC452MAC_H_
#define                     _VSC452MAC_H_

#undef HAVE_MULTICAST


/*Start Loopback */

#define BUFLEN 36
#define VSC452_ENABLE_INTERNAL_LOOPBACK  		0x01
#define VSC452_DISABLE_INTERNAL_LOOPBACK   		0x02
#define LOOPBACK_TEST					0x03

#define	VSC452IOCTL	SIOCDEVPRIVATE

/*This struct is passed between the users pace and kernel space */
struct vsc452_ioctl
{
	unsigned short cmd;                /* Command to run */
	unsigned short port;			/* Eth0/Eth1*/
	unsigned short len;                   /* lenght of the buffer */
	char  *rx_data;       			/* Pointer to the tx buffer */
	char  *tx_data; 			/* Pointer to the rx buffer */
	struct net_device_stats *app_stats; /* netstats*/
};
static int vsc452_ioctl (struct net_device *dev, struct ifreq *rq, int cmd);

/*End Loopback */
#define VSC452MAC_USE_DMA


//ring sizes
#define RX_MAX_DESC_COUNT		128
#define RX_DESC_SIZE			sizeof(struct dma_rx_desc_T)
#define RX_RING_MASK    		0x7F



#define TX_MAX_DESC_COUNT		32
#define TX_DESC_SIZE			sizeof(struct dma_tx_desc_T)
#define TX_RING_MASK    		0x1F



// Ethernet MAC Shim Registers
#define REG_TRCR    0x00    // Transmit/Receive Control Register
#define REG_TFD     0x04    // Transmit FIFO Data Register
#define REG_TFC     0x08    // Transmit FIFO Code Register
#define REG_TFAE    0x0c    // Transmit FIFO Almost Empty Register
#define REG_TFAF    0x10    // Transmit FIFO Almost Full Register
#define REG_RFD     0x14    // Receive FIFO Data Register
#define REG_RFC     0x18    // Receive FIFO Code Register
#define REG_RFAE    0x1c    // Receive FIFO Almost Empty Register
#define REG_RFAF    0x20    // Receive FIFO Almost Full Register
#define REG_IS      0x24    // Interrupt Status Register
#define REG_ISE     0x28    // Interrupt Status Enable Register
#define REG_MACC    0x2c    // MAC Control Register
#define REG_MACS    0x30    // MAC Status Register
#define REG_TSV     0x34    // Transmit Status Vector Register
#define REG_TSV2    0x38    // Transmit Status Vector 2 Register
#define REG_RSV     0x3c    // Receive Status Vector Register

//DMA Registers
#define TDCREG		0x40	//Transmit DMA Control register
#define TDDBAREG	0x44	//Transmit DMA Descriptor Base Address register
#define RDCREG		0x48	//Receive DMA Control register
#define RDDBAREG	0x4C	//Receive DMA Descriptor Base Address regster
#define RDBSREG		0x50	//Receive DMA Buffer Size register


// PE-MACMII Core Registers
#define REG_MAC1    0x80    // MAC Configuration Register 1
#define REG_MAC2    0x84    // MAC Configuration Register 2
#define REG_IPGT    0x88    // Back-to-Back Inter-Packet-Gap Register
#define REG_IPGR    0x8c    // Non-Back-to-Back Inter-Packet-Gap Register
#define REG_CLRT    0x90    // Collision Window / Retry Register
#define REG_MAXF    0x94    // Maximum Frame Register
#define REG_SUPP    0x98    // PHY Support Register
#define REG_TEST    0x9c    // Test Register
#define REG_MCFG    0xa0    // MII Mgmt Configuration Register
#define REG_MCMD    0xa4    // MII Mgmt Command Register
#define REG_MADR    0xa8    // MII Mgmt Address Register
#define REG_MWTD    0xac    // MII Mgmt Write Data Register
#define REG_MRDD    0xb0    // MII Mgmt Read Data Register
#define REG_MIND    0xb4    // MII Mgmt Indicators Register
#define REG_SA0     0xc0    // Station Address 0 Register
#define REG_SA1     0xc4    // Station Address 1 Register
#define REG_SA2     0xc8    // Station Address 2 Register


// Transmit/Receive Control Register bits
#define TRCR_RMIIROS  0x1000		 //RMII RefClk_Out Select
#define TRCR_RFR        0x20            // Receive FIFO Reset
#define TRCR_RCDR       0x10            // Receive Clock Domain Reset
#define TRCR_TFR        0x08            // Transmit FIFO Reset
#define TRCR_TCDR       0x04            // Transmit Clock Domain Reset
#define TRCR_ACSFS      0x02            // Auto-Clear Start Frame Send
#define TRCR_TSFS       0x01            // Transmit Start Frame Send
#define TRCR_DEFAULT    0x00            // TRCR Register Default Value

// Transmit FIFO Data Register bits
#define TFD_DEFAULT     0x00000000      // TFD Register Default Value

// Transmit FIFO Code Register bits
#define TFC_SOP         0x8             // Start of Packet Code
#define TFC_EOP4        0x4             // End of Packet (4 valid bytes) Code
#define TFC_EOP3        0x3             // End of Packet (3 valid bytes) Code
#define TFC_EOP2        0x2             // End of Packet (2 valid bytes) Code
#define TFC_EOP1        0x1             // End of Packet (1 valid byte) Code
#define TFC_MID         0x0             // Mid-Packet (dword) Code
#define TFC_DEFAULT     0x0             // TFC Register Default Value

// Transmit FIFO Almost Empty Register bits
#define TFAE_DEFAULT    0x200           // TFAE Register Default Value

// Transmit FIFO Almost Full Register bits
#define TFAF_DEFAULT    0x600           // TFAF Register Default Value

// Receive FIFO Data Register bits
#define RFD_DEFAULT     0x00000000      // RFD Register Default Value

// Receive FIFO Code Register bits
#define RFC_SOP         0x8             // Start of Packet Code
#define RFC_EOP4        0x4             // End of Packet (4 valid bytes) Code
#define RFC_EOP3        0x3             // End of Packet (3 valid bytes) Code
#define RFC_EOP2        0x2             // End of Packet (2 valid bytes) Code
#define RFC_EOP1        0x1             // End of Packet (1 valid byte) Code
#define RFC_MID         0x0             // Mid-Packet (dword) Code
#define RFC_DEFAULT     0x0             // RFC Register Default Value

// Receive FIFO Almost Empty Register bits
#define RFAE_DEFAULT    0x200           // RFAE Register Default Value

// Receive FIFO Almost Full Register bits
#define RFAF_DEFAULT    0x600           // RFAF Register Default Value

// Interrupt Status Register bits
#define IS_TSVP         0x20000         // Transmit Status Vector Pulse Interrupt
#define IS_TFAF         0x10000         // Transmit FIFO Almost Full Interrupt
#define IS_TFHF         0x08000         // Transmit FIFO Half Full Interrupt
#define IS_TFAE         0x04000         // Transmit FIFO Almost Empty Interrupt
#define IS_TFIR         0x02000         // Transmit FIFO Input Ready Interrupt
#define IS_TFF          0x01000         // Transmit FIFO Full Interrupt
#define IS_TCF          0x00800         // Transmitting Control Frame Interrupt
#define IS_RSVP         0x00400         // Receive Status Vector Pulse Interrupt
#define IS_RFAF         0x00200         // Receive FIFO Almost Full Interrupt
#define IS_RFHF         0x00100         // Receive FIFO Half Full Interrupt
#define IS_RFAE         0x00080         // Receive FIFO Almost Empty Interrupt
#define IS_RFOR         0x00040         // Receive FIFO Output Ready Interrupt
#define IS_RFE          0x00020         // Receive FIFO Empty Interrupt
#define IS_RGP          0x00010         // Received Good Packet Interrupt
#define IS_RFO          0x00008         // Receive FIFO Overflow Interrupt
#define IS_CRCQ         0x00004         // CRC Qualifier Interrupt
#define IS_SOF          0x00002         // Start of Frame Interrupt
#define IS_EOF          0x00001         // End of Frame Interrupt
#define IS_DEFAULT      0x00000         // IS Register Default Value
#define IS_ALL          0x3ffff         // All interrupts

// Interrupt Status Enable Register bits
#define ISE_TSVP        0x20000         // Transmit Status Vector Pulse Interrupt
#define ISE_TFAF        0x10000         // Transmit FIFO Almost Full Interrupt
#define ISE_TFHF        0x08000         // Transmit FIFO Half Full Interrupt
#define ISE_TFAE        0x04000         // Transmit FIFO Almost Empty Interrupt
#define ISE_TFIR        0x02000         // Transmit FIFO Input Ready Interrupt
#define ISE_TFF         0x01000         // Transmit FIFO Full Interrupt
#define ISE_TCF         0x00800         // Transmitting Control Frame Interrupt
#define ISE_RSVP        0x00400         // Receive Status Vector Pulse Interrupt
#define ISE_RFAF        0x00200         // Receive FIFO Almost Full Interrupt
#define ISE_RFHF        0x00100         // Receive FIFO Half Full Interrupt
#define ISE_RFAE        0x00080         // Receive FIFO Almost Empty Interrupt
#define ISE_RFOR        0x00040         // Receive FIFO Output Ready Interrupt
#define ISE_RFE         0x00020         // Receive FIFO Empty Interrupt
#define ISE_RGP         0x00010         // Received Good Packet Interrupt
#define ISE_RFO         0x00008         // Receive FIFO Overflow Interrupt
#define ISE_CRCQ        0x00004         // CRC Qualifier Interrupt
#define ISE_SOF         0x00002         // Start of Frame Interrupt
#define ISE_EOF         0x00001         // End of Frame Interrupt
#define ISE_DEFAULT     0x00000         // ISE Register Default Value
#define ISE_ALL         0x3ffff         // All interrupts

// MAC Control Register bits
#define MACC_TPT_MASK   0xffff0000      // Transmit Pause Timer Value Bits
#define MACC_TPCF       0x00000020      // Transmit Pause Control Frame
#define MACC_THDFC      0x00000010      // Transmit Half-Duplex Flow Control
#define MACC_PPHE       0x00000008      // Per-Packet Huge Enable
#define MACC_PPPE       0x00000004      // Per-Packet Pad Enable
#define MACC_PPCE       0x00000002      // Per-Packet CRC Enable
#define MACC_PPO        0x00000001      // Per-Packet Override
#define MACC_DEFAULT    0x00000000      // MACC Register Default Value

// MAC Status Register bits
#define MACS_SRE        0x1000          // Synchronized Receive Enable
#define MACS_UPI        0x0800          // Unicast Packet Indicator
#define MACS_MPI        0x0400          // Multicast Packet Indicator
#define MACS_BPI        0x0200          // Broadcast Packet Indicator
#define MACS_CRC_MASK   0x01ff          // CRC Output Bits
#define MACS_DEFAULT    0x0000          // MACS Default Value

// Transmit Status Vector Register bits
#define TSV_TU          0x80000000      // Transmit Underrun
#define TSV_TG          0x40000000      // Transmit Giant
#define TSV_TLC         0x20000000      // Transmit Late Collision
#define TSV_TMC         0x10000000      // Transmit Maximum Collisions
#define TSV_TER         0x08000000      // Transmit Excessive Defer
#define TSV_TPD         0x04000000      // Transmit Packet Defer
#define TSV_TB          0x02000000      // Transmit Broadcast
#define TSV_TM          0x01000000      // Transmit Multicast
#define TSV_TD          0x00800000      // Transmit Done
#define TSV_TLOR        0x00400000      // Transmit Length Out-of-Range
#define TSV_TLCE        0x00200000      // Transmit Length Check Error
#define TSV_TCE         0x00100000      // Transmit CRC Error
#define TSV_TCC_MASK    0x000f0000      // Transmit Collision Count Bits
#define TSV_TBC_MASK    0x0000ffff      // Transmit Byte Count Bits
#define TSV_DEFAULT     0x00000000      // TSV Register Default Value

// Transmit Status Vector 2 Register bits
#define TSV2_TVTF       0x80000         // Transmit VLAN Tagged Frame
#define TSV2_BPA        0x40000         // Backpressure Applied
#define TSV2_TPCF       0x20000         // Transmit Pause Control Frame
#define TSV2_TCF        0x10000         // Transmit Control Frame
#define TSV2_TB_MASK    0x0ffff         // Total Bytes Transmitted Bits
#define TSV2_DEFAULT    0x00000         // TSV2 Register Default Value

// Receive Status Vector Register bits
#define RSV_RVTD        0x40000000      // Receive VLAN Type Detected
#define RSV_UOC         0x20000000      // Receive Unsupported OpCode
#define RSV_RPCF        0x10000000      // Receive Pause Control Frame
#define RSV_RCF         0x08000000      // Receive Control Frame
#define RSV_DN          0x04000000      // Dribble Nibble
#define RSV_BP          0x02000000      // Broadcast Packet
#define RSV_MP          0x01000000      // Multicast Packet
#define RSV_ROK         0x00800000      // Receive OK
#define RSV_LOR         0x00400000      // Length Out-of-Range
#define RSV_LCE         0x00200000      // Length Check Error
#define RSV_CE          0x00100000      // CRC Error
#define RSV_RCV         0x00080000      // Receive Code Violation
#define RSV_CEPS        0x00040000      // Carrier Event Previously Seen
#define RSV_REPS        0x00020000      // RXDV Event Previously Seen
#define RSV_PPI         0x00010000      // Packet Previously Ignored
#define RSV_RBC_MASK    0x0000ffff      // Received Byte Count Bits

// MAC Configuration Register 1 bits
#define MAC1_SOR        0x8000          // Soft Reset
#define MAC1_SIR        0x4000          // Simulation Reset
#define MAC1_PEMCSRR    0x0800          // PEMCS/Rx Reset
#define MAC1_PERFUNR    0x0400          // PERFUN Reset
#define MAC1_PEMCSTR    0x0200          // PEMCS/Tx Reset
#define MAC1_PETFUNR    0x0100          // PETFUN Reset
#define MAC1_LB         0x0010          // Loopback
#define MAC1_TFC        0x0008          // Tx Flow Control
#define MAC1_RFC        0x0004          // Rx Flow Control
#define MAC1_PARF       0x0002          // Pass All Receive Frames
#define MAC1_RE         0x0001          // Receive Enable
#define MAC1_DEFAULT    MAC1_SOR        // MAC1 Register Default Value

// MAC Configuration Register 2 bits
#define MAC2_ED         0x4000          // Excess Defer
#define MAC2_BPNB       0x2000          // Backpressure / No Backoff
#define MAC2_NB         0x1000          // No Backoff
#define MAC2_LPE        0x0200          // Long Preamble Enforcement
#define MAC2_PPE        0x0100          // Pure Preamble Enforcement
#define MAC2_ADPE       0x0080          // Auto-Detect Pad Enable
#define MAC2_VPE        0x0040          // VLAN Pad Enable
#define MAC2_PCE        0x0020          // Pad / CRC Enable
#define MAC2_CE         0x0010          // CRC Enable
#define MAC2_DC         0x0008          // Delayed CRC
#define MAC2_HFE        0x0004          // Huge Frame Enable
#define MAC2_FLC        0x0002          // Frame Length Checking
#define MAC2_FD         0x0001          // Full-Duplex
#define MAC2_DEFAULT    0x0000          // MAC2 Default Value

// Back-to-Back Inter-Packet-Gap Register bits
#define IPGT_MASK       0x007f          // Mask
#define IPGT_DEFAULT    0x0000          // IPGT Default Value

// Non-Back-to-Back Inter-Packet-Gap Register bits
#define IPGR_1_MASK     0x7f00          // Part 1 bits
#define IPGR_2_MASK     0x007f          // Part 2 bits
#define IPGR_DEFAULT    0x0000          // IPGR Default Value

// Collision Window / Retry Register bits
#define CLRT_CW_MASK    0x3f00          // Collision Window Bits
#define CLRT_RM_MASK    0x000f          // Retransmission Maximum Bits
#define CLRT_DEFAULT    0x370f          // CLRT Default Value

// Maximum Frame Register bits
#define MAXF_DEFAULT    0x0600          // MAXF Default Value

// PHY Support Register bits
#define SUPP_RIM        0x8000          // Reset Interface Module
#define SUPP_PM         0x1000          // PHY Mode
#define SUPP_RPERMII    0x0800          // Reset PERMII
#define SUPP_SPEED      0x0100          // Speed
#define SUPP_RPE100X    0x0080          // Reset PE100X
#define SUPP_FC         0x0040          // Force Quiet
#define SUPP_NC         0x0020          // No Cipher
#define SUPP_DLF        0x0010          // Disable Link Fail
#define SUPP_RPE10T     0x0008          // Reset PE10T
#define SUPP_EJP        0x0002          // Enable Jabber Protection
#define SUPP_BM         0x0001          // Bit Mode
#define SUPP_DEFAULT    SUPP_PM         // SUPP Default Value

// Test Register bits
#define TEST_TBP        0x0004          // Test Backpressure
#define TEST_TP         0x0002          // Test Pause
#define TEST_SPQ        0x0001          // Shortcut Pause Quanta
#define TEST_DEFAULT    0x0000          // TEST Default Value

// MII Mgmt Configuration Register bits
#define MCFG_RMM        0x8000          // Reset MII Management Module
#define MCFG_CS_MASK    0x001c          // Clock Select bits
#define MCFG_SP         0x0002          // Suppress Preamble
#define MCFG_SI         0x0001          // Scan Increment
#define MCFG_DEFAULT    0x0000          // MCFG Default Value

// MII Mgmt Command Register bits
#define MCMD_SCAN       0x0002          // Scan
#define MCMD_READ       0x0001          // Read
#define MCMD_DEFAULT    0x0000          // MCMD Default Value

// MII Mgmt Address Register bits
#define MADR_PHY_MASK   0x1f00          // PHY Address bits
#define MADR_PEG_MASK   0x001f          // Register Address bits
#define MADR_DEFAULT    0x0000          // MADR Default Value

// MII Mgmt Write Data Register bits
#define MWTD_DEFAULT    0x0000          // MWTD Default Value

// MII Mgmt Read Data Register bits
#define MRDD_DEFAULT    0x0000          // MRDD Default Value

// MII Mgmt Indicators Register bits
#define MIND_LF         0x0008          // Link Fail
#define MIND_NV         0x0004          // Not Valid
#define MIND_SCAN       0x0002          // Scanning
#define MIND_BUSY       0x0001          // Busy
#define MIND_DEFAULT    0x0000          // MIND Default Value

// Station Address 0 Register bits
#define SA0_1_MASK      0xff00          // 1st Octet
#define SA0_2_MASK      0x00ff          // 2nd Octet
#define SA0_DEFAULT     0x0000          // SA0 Default Value

// Station Address 1 Register bits
#define SA1_1_MASK      0xff00          // 1st Octet
#define SA1_2_MASK      0x00ff          // 2nd Octet
#define SA1_DEFAULT     0x0000          // SA0 Default Value

// Station Address 2 Register bits
#define SA2_1_MASK      0xff00          // 1st Octet
#define SA2_2_MASK      0x00ff          // 2nd Octet
#define SA2_DEFAULT     0x0000          // SA0 Default Value


//Transmit DMA control
#define TDE				0x80000000				//Tx DMA Enable
#define TDR				0x40000000				//Tx DMA Resume
#define TDSMASK			0x00030000				//transmit DMA state
#define TADMASK			0x0000FFFF				//Transmit Active Descriptor

//transmit DMA descriptor Base Address
#define TDBAMASK		0x0FFFFFFF				//Tx Descriptor base Address

//receive DMA control
#define RDE				0x80000000				//Rx DMA Enable
#define RDR				0x40000000				//Rx DMA resume
#define TBPM			0x20000000				//Two byte pad mode
#define RDSMASK			0x00030000				//Receive DMA state
#define RADMASK			0x0000FFFF				//Receive active descriptor

//receive DMA descriptor base Address
#define RDBAMASK		0x0FFFFFFF				//Rx Descriptor base Address

//receive DMA buffer size
#define RBSMASK			0x0000FFFF				//RX DMA Buffer size

// MAC Interrupt Status and Enable constants
#define PHYINTR			0x01000000				// PHY Interrupt
#define TPCINTR			0x00800000				// Transmit DMA complete Interrupt
#define TXQOINTR		0x00400000				// Transmit Queue Overrun Interrupt
#define RPCINTR			0x00200000				// Receive packet complete Interrupt
#define RQOINTR			0x00100000				// Receive Queue Overrun Interrupt
#define RBOIINTR		0x00080000				// Receive buffer overrun Interrupt
#define RFOIINTR		0x00040000				// Receive FIFO overrun Interrupt
#define TSVPIINTR		0x00020000				// Transmit Status Vector Pulse Interrupt
#define TFAFIINTR		0x00010000				// Transmit FIFO almost FULL interrupt
#define TFHFIINT		0x00008000				// Transmit FIFO half full Interrupt
#define TFAEIINTR		0x00004000				// Transmit FIFO almost empty Interrupt
#define TFIRIINTR		0x00002000				// Transmit FIFO Input Ready Interrupt
#define TFFIINTR		0x00001000				// Transmit FIFO Full Interrupt
#define TCFIINTR		0x00000800				// Transmit control frame interrupt
#define RSVPIINTR		0x00000400				// Receive status Vector pulse Interrupt
#define RFAFIINTR		0x00000200				// Receive FIFO almost full Interrupt
#define RFHFIINTR		0x00000100				// Receive FIFO half full Interrupt
#define RFAEIINTR		0x00000080				// Receive FIFO almost emptry Interrupt
#define RFORIINTR		0x00000040				// Receive FIFO output ready Interrupt
#define RFEIINTR		0x00000020				// Receive FIFO empty Interrupt
#define RGPIINTR		0x00000010				// Received Good Packet Interrupt
#define RFOFINTR		0x00000008				// Received FIFO Overflow Interrupt
#define CQIINTR			0x00000004				// CRC Qualified Interrupt
#define SOFIINTR		0x00000002				// Start of Frame Interrupt
#define EOFIINTR		0x00000001				// End of frame Interrupt



#define OWNER_HOST  0x0
#define OWNER_DMA   0x1
#define TRUE		0x1
#define FALSE 		0x0

//investigate
#define TX_THRESHOLD      0x10

/*check whether we can use bit fields*/
#if !defined(__LITTLE_ENDIAN_BITFIELD) && !defined(__BIG_ENDIAN_BITFIELD)
#error "Adjust <asm/byteorder.h> defines"
#endif

/* typedefs */
typedef unsigned long uint32;
typedef unsigned short uint16;
typedef unsigned char uint8;



#ifdef VSC452MAC_USE_DMA
//receive DMA descriptor as givein in section 4.3.1.3.1 of maxim document
//dma rx decriptor
struct dma_rx_desc_T
{
    uint32 owner:1;            /* 0 = Host, 1 = DMA */
    uint32 vlan:1;
    uint32 unsupported:1;
    uint32 pauseframe:1;
    uint32 ctrlframe:1;
    uint32 dnb:1;   		/* dribble nibble*/
    uint32 broadcast:1;
    uint32 multicast:1;
    uint32 ok:1;
    uint32 lenouterr:1;
    uint32 len_chkerr:1;
    uint32 crcerr:1;
    uint32 codviol:1;
    uint32 prvcarrier:1;
    uint32 prvrxdv:1;
    uint32 prv_pktign:1;
    uint32 rxcnt:16;

    uint32 eoq:1;
    uint32 rsrvd1:2;
    uint32 orunerr:1;
    uint32 rxaddr:28;

    uint32 rsrvd2:16;
    uint32 fifo_rxcnt:16;
    uint32 rsrvd3;
};


//transmit DMA descriptor as givein in section 4.3.1.2.1 of maxim document
//dma tx descriptor
struct dma_tx_desc_T
{
    uint32 owner:1;            // Desc owner
    uint32 intrenable:1;
    uint32 rsrvd1:5;
    uint32 pad:1;
    uint32 pp_paden:1;
    uint32 pp_crcen:1;
    uint32 pp_hugeen:1;
    uint32 pp_override:1;
    uint32 vlan_frame:1;
    uint32 backpres:1;
    uint32 pauseframe:1;
    uint32 ctrlframe:1;
    uint32 urun:1;
    uint32 giant:1;
    uint32 latecol:1;
    uint32 maxcol:1;
    uint32 exdefer:1;
    uint32 defer:1;
    uint32 broadcast:1;
    uint32 multicast:1;
    uint32 done:1;
    uint32 lenouterr:1;
    uint32 lenchkerr:1;
    uint32 crcerr:1;
    uint32 colcnt:4;

    uint32 eoq:1;
    uint32 dw1_rsvd:3;
    uint32 txaddr:28;

    uint32 tsv_txcnt:16;
    uint32 pkt_txcnt:16;
    uint32 rsrvd2;

};
#endif //VSC452MAC_USE_DMA

struct vscmac_dma_T
{
	struct sk_buff  * sk_buf;
	dma_addr_t 		skb_dma;
	int len;

};


#define SPEED_100           1
#define SPEED_10            0
#define DUPLEX_FULL         1
#define DUPLEX_HALF         0
#define LINK_UP             1
#define LINK_DOWN           0

struct vsc452mac_local
{
    // these are things that the kernel wants me to keep, so users
    // can find out semi-useless statistics of how well the card is
    // performing
    struct net_device_stats stats;

    // Address of our PHY port
    unsigned int		phyaddr;
	unsigned int 		port;

    // Type of PHY
    unsigned int		phytype;

    // Last contents of PHY Register 18
    unsigned int		lastPhy18;

    spinlock_t          lock;

	struct work_struct	rcv_work;

	volatile	int 	trans_busy;
	unsigned int 		intrstatus;
    unsigned int 		ierstatus;
    struct timer_list dma_timer;

#ifdef VSC452MAC_USE_DMA
	struct dma_rx_desc_T	*rx_ring_desc;		/*  rx desc */
	struct dma_tx_desc_T	*tx_ring_desc;		/* tx desc */

	dma_addr_t		rx_ring_dma;             /* rx dma base address         */
    dma_addr_t		tx_ring_dma; 			/*   tx dma address    		 */

    unsigned int 	rx_front;                 /*  rx queue front       */
	unsigned int 	rx_rear;               /*  rx queue rear */

	unsigned int 	tx_front;                /* tx queue front */
	unsigned int 	tx_rear; 			   /* tx queue rear */

	struct vscmac_dma_T	rx_buf[RX_MAX_DESC_COUNT];			/* rx buffer */
	struct vscmac_dma_T	tx_buf[TX_MAX_DESC_COUNT];			/* tx buffer placeholder */



#endif
	struct napi_struct napi;
       struct net_device *dev;
};


static irqreturn_t  vsc452mac_interrupt(int irq, void * dev_id);
static int vsc452mac_open(struct net_device *dev);
#if 0
static void vsc452mac_timeout (struct net_device *dev);
#endif
static int vsc452mac_close(struct net_device *dev);
static struct net_device_stats * vsc452mac_query_statistics( struct net_device *dev);
#ifdef HAVE_MULTICAST
static void vsc452mac_set_multicast_list(struct net_device *dev);
#endif
static void vsc452mac_phy_configure(struct net_device* dev);
//static void vsc452mac_rcv(void *dev);
static int do_vsc452mac_probe(struct net_device *dev);
static void vsc452mac_reset( struct net_device* dev );

//static void vsc452mac_enable( struct net_device *dev );
#if 0
static unsigned short vsc452mac_read_phy_register(unsigned int ioaddr, unsigned char phyaddr, unsigned char phyreg);
static void vsc452mac_write_phy_register(unsigned int ioaddr, unsigned char phyaddr, unsigned char phyreg, unsigned short phydata);
#endif
static void vsc452mac_phy_configure(struct net_device* dev);


#ifdef VSC452MAC_USE_DMA
static void vsc452mac_init_dma(struct net_device *dev);
#endif



#endif
