
/*
 * Copyright (c) 2009 - Network Appliance Inc.
 *
 */

/*********************************************************************
 * VSC452 driver module for SPI manual mode interface
 *
 *********************************************************************/
#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/poll.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/wait.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/cacheflush.h>
#include <asm/ioctl.h>

#include "spi_vsc.h"

/*****Prototye Functions*************************/
static  int  spi_vsc_init (void );
static void  spi_vsc_exit (void);
static  int  spi_vsc_open (struct inode *, struct file *);
static  int  spi_vsc_release (struct inode *, struct file *);
static  int  spi_vsc_read (struct file *, char *, size_t, loff_t *);
static  int  spi_vsc_write (struct file *, const char *, size_t, loff_t *);
static  int  spi_vsc_ioctl (struct inode *, struct file *, uint, ulong);



static struct file_operations spi_vsc_ops;
static void  *spi_dev;

int __init  
spi_vsc_init(void)
{
	int  err, ret=0;
	uint32_t spi_base;  
	void *dev= &spi_dev; 

	
	printk(" VSC452 SPI Init \n ");

	memset(&spi_vsc_ops,0,sizeof(spi_vsc_ops));
	spi_vsc_ops.ioctl   = spi_vsc_ioctl;
	spi_vsc_ops.open    = spi_vsc_open;
	spi_vsc_ops.release = spi_vsc_release;
	spi_vsc_ops.read    = spi_vsc_read;
	spi_vsc_ops.write   = spi_vsc_write;
	spi_vsc_ops.owner   = THIS_MODULE;

	/*********Register Character Device ********************/

	if ((err = register_chrdev(SPI_MAJOR_NO, SPI_NAME, &spi_vsc_ops)) < 0)
	{
		printk ("failed to register device (err %d)\n", err);
		ret = -ENODEV;
		goto fail;
	}  else {
		printk ("registered device %s)\n", SPI_NAME);
	}

	/* Remap registers */
	spi_base = SPI_BASE_OFFSET;
	if ((dev = ioremap_nocache(spi_base, SPI_SIZE)) == NULL) 
	{
		printk("failed to map SPI IO space %08x-%08x to memory",spi_base, spi_base + SPI_SIZE - 1);
		ret = -EIO;
		goto fail;
	}

	/* check spi memory region */
	if ((err = check_mem_region(spi_base, SPI_SIZE)) < 0)  
	{
		printk("SPI IO space %08x-%08x already in use (err %d)",spi_base, spi_base + SPI_SIZE - 1, err);
		ret = -EBUSY;
		goto fail;
	}

	/* request for the memory region */	
	if (request_mem_region(spi_base, SPI_SIZE, SPI_NAME)  == NULL) 
	{ 
		printk("IO space %08x-%08x spi request mem region failed (err %d)",spi_base, spi_base + SPI_SIZE - 1, err);
		ret = -ENODEV;
		goto fail;

	}

	printk("SPI Module was sucessully loaded \n");
	return ret;

fail:
	spi_vsc_exit();
	return ret;
  
}

	
static void  
spi_vsc_exit(void)
{
	void *dev= &spi_dev; 
	uint32_t spi_base;

	printk(" VSC452 SPI Exit \n ");
	spi_base = SPI_BASE_OFFSET;

	/* release the memory occupied */
	release_mem_region(spi_base, SPI_SIZE); 

	/* unregister the device */
	unregister_chrdev(SPI_MAJOR_NO, SPI_NAME);

	/* Character Device unmapping   */ 
	if (dev) 
	{
		iounmap(dev); 
		dev = NULL;
	}

	printk ( "SPI Driver is unloaded sucessfully.\n");
	return;
	
 
}

/*iowrite ****/
inline void spi_write_reg(unsigned long addr, unsigned long val)
{
        *((volatile uint32_t*)addr) = val;
}

/*********ioread*************/
inline unsigned long spi_read_reg(unsigned long addr )
{
        return ((*(volatile uint32_t*)addr));
}


static  int
spi_vsc_open (struct inode *inode, struct file *file)
{
	file->f_op =&spi_vsc_ops;
      	return 0;
}

static  int 
spi_vsc_release (struct inode *inode,struct file *file)
{
	return 0;
}

static  int  
spi_vsc_read (struct file *file, char *buffer,size_t size, loff_t *offset)
{
	return 0;
}

static  int  
spi_vsc_write (struct file *file, const char *buffer, 
size_t size, loff_t * offset)
{
	return 0;
}

static int  
spi_vsc_ioctl (struct inode *inode, struct file *file, uint cmd , ulong arg)
{
	unsigned int val;
	int i = 0, count;
	spi_ioctl_t pkt;

        switch ( cmd )
        {
		case SPI_READ_REG:
        		if (copy_from_user((void*)&pkt, 
						(void*)arg, 
						sizeof(spi_ioctl_t))) {
				return -EFAULT;
			}
			READ_REG(pkt.reg, val);
			pkt.data = val;
//			SPI_DEBUG_PRINT("Read", val, pkt.reg);

			if (copy_to_user((void*)arg, 
						(void*)&pkt, 
						sizeof(spi_ioctl_t))) {
				return -EFAULT;
			}
                        break;
		case SPI_WRITE_REG:
        		if (copy_from_user((void*)&pkt, 
						(void*)arg, 
						sizeof(spi_ioctl_t))) {
				return -EFAULT;
			}
			WRITE_REG(pkt.reg, pkt.data);
//			SPI_DEBUG_PRINT("Set", pkt.data, pkt.reg);
			break;
		default:
			break;
	}
	return 0;
}

module_init(spi_vsc_init);
module_exit(spi_vsc_exit);

MODULE_DESCRIPTION("VSC452 SPI Driver");
MODULE_AUTHOR("NetApp Inc");
MODULE_LICENSE("GPL");


	


