#include <common.h>
#include <command.h>
#include <asm/mipsregs.h>

//extern unsigned long ExceptArg;

const char *ExceptionName[] =
{
	"Interrupt",
	"TLB Modification Error",
	"TLB Load or Fetch Error",
	"TLB Store Error",
	"Address Load or Fetch Error",
	"Address Store Error",
	"Bus Load or Fetch Error",
	"Bus Store Error",
	"System Call",
	"BreakPoint",
	"Reserved 10",
	"Coprocessor Unusable",
	"Arithmetic Overflow",
	"Trap Exception",
	"Reserved 14",
	"Reserved 15",
	"Implement Specific 1",
	"Implement Specific 2",
	"Coprocessor 2 Exception",
	"Reserved 19",
	"Reserved 20",
	"Reserved 21",
	"Reserved 22",
	"Watch Reference",
	"Machine Check",
	"Reserved 25",
	"Reserved 26",
	"Reserved 27",
	"Reserved 28",
	"Reserved 29",
	"Reserved 30",
	"Reserved 31",
};

void 
show_regs(struct pt_regs *regs)
{
	const int field = 2 * sizeof(unsigned long);
	unsigned int cause = regs->cp0_cause;
	int i;

	/*
	 * Saved main processor registers
	 */
	for (i = 0; i < 32; ) {
		if ((i % 4) == 0)
			printf("$%2d   :", i);
		if (i == 0)
			printf(" %0*lx", field, 0UL);
		else if (i == 26 || i == 27)
			printf(" %*s", field, "");
		else
			printf(" %0*lx", field, regs->regs[i]);

		i++;
		if ((i % 4) == 0)
			printf("\n");
	}

	printf("Hi    : %0*lx\n", field, regs->hi);
	printf("Lo    : %0*lx\n", field, regs->lo);

	/*
	 * Saved cp0 registers
	 */
	printf("epc   : %0*lx \n", field, regs->cp0_epc);
	printf("Status: %08x    \n", (uint32_t) regs->cp0_status);

	if (regs->cp0_status & 0x10) 
		printf("USER ");
	else
		printf("KERNEL ");
	if (regs->cp0_status & ST0_ERL)
		printf("ERL ");
	if (regs->cp0_status & ST0_EXL)
		printf("EXL ");
	if (regs->cp0_status & ST0_IE)
		printf("IE ");
	printf("\n");

	printf("Cause : %08x\n", cause);

	cause = (cause & CAUSEF_EXCCODE) >> CAUSEB_EXCCODE;
	if (1 <= cause && cause <= 5)
		printf("BadVA : %0*lx\n", field, regs->cp0_badvaddr);
	printf("\n");
	return;
}

void
reserved_handler(struct pt_regs *regs)
{
	unsigned long cause;
	//printf("FATAL: Reserved Exception from Index 0x%lx !!!!\n",ExceptArg);
	printf("FATAL: Reserved Exception!!!!\n");

	cause = (regs->cp0_cause & 0x7C) >> 2;
	printf("Exception Cause : %s\n",ExceptionName[cause]);
	show_regs(regs);
	printf("System Halted\n");
	while (1);
	return;
}

extern void HandleIntr(unsigned long IntrStatus);


void
trap_handler(struct pt_regs *regs)
{
	unsigned long cause;
	
	cause = (regs->cp0_cause & 0x7C) >> 2;

	/* If cause is 0 , then it is a interrupt */
	if (cause == 0)
	{
		printf("Interrupt\n");
		HandleIntr((regs->cp0_cause & 0xFF00) >> 8);
		return ;
	}

	//printf("Exception Generated from Offset 0x%lx\n",ExceptArg);
	printf("Exception Generated\n");
	printf("Exception Cause : %s\n",ExceptionName[cause]);
	show_regs(regs);
	printf("System Halted\n");
	while (1);
	return;
}



