/*
 * $Id$
 *
 * Copyright 2008, Network Appliance Inc.
 * Author: Jason McMullan <mcmullan@netapp.com>
 *
 */

#ifndef ASM_MIPS_ARCH_VSC452_I2C_H
#define ASM_MIPS_ARCH_VSC452_I2C_H

#define ARRAY_SIZE(x) (sizeof(x) /sizeof((x)[0]))

#define VSC452_I2C_INTERFACES	6
#define VSC452_I2C_FIFO_MAX	80	/* 80 byte FIFO */

#define I2C_BASE(x)	(0x10200500 + (0x80 * (x)))

#define SID		0x00	/* Serial Interface Data */
#define SID_DATA(x)		((x) & 0xff)	/* Data */
#define SID_SCA_MASTER		(1 << 7)	/* Master-only interface */
#define SID_SCA_ADDR(x)		((x) & 0x7f)	/* Slave address of interface */
#define SID_SSCA_ENABLE		(1 << 7)	/* Enable SSCA */
#define SID_SSCA_ADDR(x)	((x) & 0x7f)	/* Secondary Slave address of interface */
#define SID_CMS_NAS		(1 << 7)	/* Don't snoop the bus */
#define SID_CMS_CLK_98KHZ	(0 << 0)	/* Clock Select */
#define SID_CMS_CLK_78KHZ	(1 << 0)	/* Clock Select */
#define SID_CMS_CLK_347KHZ	(2 << 0)	/* Clock Select */
#define SID_CMS_CLK_284KHZ	(3 << 0)	/* Clock Select */
#define SID_CMS_CLK_417KHZ	(4 << 0)	/* Clock Select */
#define SID_CMS_CLK_340KHZ	(5 << 0)	/* Clock Select */
#define SID_CMS_CLK_1042KHZ	(6 << 0)	/* Clock Select */
#define SID_CMS_CLK_852KHZ	(7 << 0)	/* Clock Select */

#define SIC		0x04	/* Serial Interface Control */
	/* Read Mode */
#define SIC_PIN		(1 << 7)		/* Pending Interrupt Not */
#define SIC_SSN		(1 << 6)		/* Start Status Not */
#define SIC_STS		(1 << 5)		/* Stop Status */
#define SIC_BER		(1 << 4)		/* Bus Error */
#define SIC_LRB		(1 << 3)		/* Last Received Bit */
#define SIC_AAS		(1 << 2)		/* Addressed as Slave */
#define SIC_LAB		(1 << 1)		/* Lost Arbitaration */
#define SIC_BBN		(1 << 0)		/* Bus Busy Not */
	/* Write Mode */
#define SIC_ES_SCA	(0 << 4)		/* SID is Slave Compare Address */
#define SIC_ES_SSCA	(7 << 4)		/* SID is Secondary Slave Compare Address */
#define SIC_ES_DTH	(1 << 4)		/* SID is Data Out Temporary Holding Reg */
#define SIC_ES_CMS	(2 << 4)		/* SID is Clock and Mode selection */
#define SIC_ES_DATA	(4 << 4)		/* SID is 8-byte data in/out */
#define SIC_ENI		(1 << 3)		/* Enable Interrupt */
#define SIC_STA		(1 << 2)		/* Send Start */
#define SIC_STO		(1 << 1)		/* Send Stop */
#define SIC_ACK		(1 << 0)		/* Acknowledge */

#define SIBC		0x08	/* Serial Interface Bit Control */
#define SIBC_SISR	(1 << 7)		/* Soft Reset */
#define SIBC_SDIE	(1 << 6)		/* Stop Detected Interrupt Enable */
#define SIBC_SCC	(1 << 1)		/* Serial Clock Control */
#define SIBC_SDC	(1 << 0)		/* Serial Data Control */

#define SIBS		0x0c	/* Serial Interface Bit Status */
#define SIBS_SIIS	(1 << 7)		/* Serial Interface Interrupt Status */
#define SIBS_SDIS	(1 << 6)		/* Stop Detected Interrupt Status */
#define SIBS_SCS	(1 << 1)		/* Serial Clock Status */
#define SIBS_SDS	(1 << 0)		/* Serial Data Status */

#define SIFC		0x10	/* Serial Interface RX FIFO Control */
#define SIFC_SRFE	(1 << 7)		/* Slave RX FIFO Enable */
#define SIFC_CRFP	(1 << 6)		/* Clear RX FIFO Pointers */
#define SIFC_STMIE	(1 << 1)		/* Slave TX Mode Interrupt Enable */
#define SIFC_RFIE	(1 << 0)		/* RX FIFO Interrupt Enable */

#define SIFS		0x14	/* Serial Interface RX FIFO Status */
#define SIFS_RFF	(1 << 3)		/* RX FIFO Full */
#define SIFS_RFE	(1 << 2)		/* RX FIFO Empty */
#define SIFS_STMIS	(1 << 1)		/* Slave Transmit Mode Interrupt Status */
#define SIFS_RFIS	(1 << 0)		/* Receive FIFO Interrupt Status */

#define SIFD		0x18	/* Serial Interface RX FIFO Data */
#define SIFD_FD(x)	((x) & 0xff)		/* FIFO Data */

#endif /* ASM_MIPS_ARCH_VSC452_I2C_H */

