/*****************************************************************************
** vsc452.h : memory/register map for VSC452 chip
*****************************************************************************/

/*
** $Revision: 1.7 $
** $Date: 2006/03/07 00:02:10 $
** $Author: jsmith $
*/

#ifndef __VSC452_H_
#define __VSC452_H_

/*
** chip ID and revisions (JPS: different register source for these)
** last nibble is the rev #
*/
#define VSC452_REVISION_FPGA        (0x00045200)
#define SYS_REVISION                VSC452_REVISION_FPGA


/* CPU interrupt offsets */
#define MIPSCPU_INT_SW0		0
#define MIPSCPU_INT_SW1		1

#define MIPSCPU_INT_ETH0		3		
#define MIPSCPU_INT_ETH1 		3	
#define MIPSCPU_INT_CPUCTR		5		
#define MIPSCPU_INT_UART_GPIO 	6
/*****************************************************************************
** CPU-Specific Interrupt Assignments
*****************************************************************************/
#define MIPSCPU_INT_BASE 	0

#define SYS_CAUSE_SWI0          V3000_INT_SW0
#define SYS_CAUSE_SWI1          V3000_INT_SW1

#define BMC_CAUSE_SYSIF         V3000_INT_0        // KCS/SMIC
#define SYS_CAUSE_TRANSPT       BMC_CAUSE_SYSIF

#define SYS_CAUSE_TIMER         V3000_INT_3        // timer

#define SYS_CAUSE_INT0          V3000_INT_4        // GPIO's
#define BMC_CAUSE_GPIO          V3000_INT_4
#define SYS_CAUSE_SERIALICE     V3000_INT_4
#define SYS_CAUSE_GPUART        V3000_INT_4

#define SYS_CAUSE_GTWS          V3000_INT_5

#define SYS_CAUSE_INTMASK       V3000_INTMASK

#define SYS_CAUSE_EXCINT        (0x0)       /* exception field: interrupt */
#define SYS_CAUSE_EXCBREAK      (0x9)       /* exception field: break */

#define SYS_STATUS_IEC          (0x0001)    /* global interrupt enable */

#define SYS_CLOCK_RATE           (300000000)  /* CPU clock 300MHz ?? */


/*****************************************************************************
** Memory Map
*****************************************************************************/

#define SYS_CNTL_BASE       (0xB0200000)        /* Register base */

//
///*
//** base locations of MAC instance control registers (see pemac.h for offsets)
//*/
#define SYS_MAC0_BASE       (SYS_CNTL_BASE + 0x200000)
#define SYS_MAC1_BASE       (SYS_CNTL_BASE + 0x280000)

// This just creates a 32 bit integer constant, you supply the "0x" in the msb arg to make it hex
#define ADDR32(msb,b1,b2,lsb) msb##b1##b2##lsb
// Size in kilobytes
#define ADDR_KB(kb) 1024*kb
// Size in megabytes
#define ADDR_MB(mb) 1024*1024*mb

// TODO: The idea here was to define the CHIP, not the configuration, that should be done in a different file
// but instead of being elegant, I'm going to hack it for now since this is just a proof of concept project.
// 
//// Base address of the chip select: "parallel chip select #0"
//#define VSC452_PCS0 		ADDR32(0x1C,00,00,00)
//// Size of the chip on the chip select region -- not the size of the chip, total addressable size of the CS
//#define VSC452_PCS0_SIZE 	ADDR_MB(64)
//
//#define VSC452_PCS1 		ADDR32(0x18,00,00,00)
//#define VSC452_PCS1_SIZE 	ADDR32_MB(64)
//
//#define VSC452_PCS2 		ADDR32(0x18,00,00,00)
//#define VSC452_PCS2_SIZE 	ADDR32_MB(64)

// Simple config for parallel -- note that the sizes of the RAM/ROM parts don't really belong in the chip definition file
#define ROM_BASE			ADDR32(0x1f,c0,00,00)
#define ROM_SIZE			ADDR_MB(4)

#define RAM0_BASE			ADDR32(0x12,80,00,00)
#define RAM0_SIZE			ADDR_MB(8)

#define RAM1_BASE			ADDR32(0x13,00,00,00)
#define RAM1_SIZE			ADDR_MB(8)

#define DDR_BASE			ADDR32(0x00,00,00,00)
#define DDR_SIZE			ADDR_MB(32)

/*****************************************************************************
******************************************************************************
**                              REGISTERS
******************************************************************************
*****************************************************************************/

/*
** JPS: Provisional, just to get a clean compile.  This register
**      might not exist, but it is only used during firmware upgrades.
**      At least, BASE+0x850 does not conflict with any actual register.
*/
#define SOFT_RESET
#define SYS_SOFT_RESET      (SYS_CNTL_BASE + 0x850)


/*****************************************************************************
** Parallel Flash Control : in theory, the code will never actually use
**                          these values, since the flash is ALWAYS SPI.
*****************************************************************************/

#define SYS_RCSC       (SYS_CNTL_BASE + 0x000)  // PCSC0 /* ROM CS control reg */
#define SYS_RCSWC      (SYS_CNTL_BASE + 0x010)  // PCSWC /* ROM CS write enable reg */

#define SYS_CSC0       (SYS_CNTL_BASE + 0x004)  // PCSC1 /* CS0 timing contrl */
#define SYS_PCSC1      (SYS_CNTL_BASE + 0x004)  // CS1 control
#define SYS_PCSC2      (SYS_CNTL_BASE + 0x008)  // CS2 control
#define SYS_PCSC3      (SYS_CNTL_BASE + 0x00C)  // CS3 control

#define SYS_RCSWC_BF_8k     (0x00)              /* 8kb BFW memsize */
#define SYS_RCSWC_BF_16k    (0x01)              /* 16kb BFW memsize */
#define SYS_RCSWC_BF_32k    (0x02)              /* 32kb BFW memsize */
#define SYS_RCSWC_BF_64k    (0x03)              /* 64kb BFW memsize */
#define SYS_RCSWC_BF_128k   (0x04)              /* 128kb BFW memsize */
#define SYS_RCSWC_BF_256k   (0x05)              /* 256kb BFW memsize */
#define SYS_RCSWC_BF_512k   (0x06)              /* 512kb BFW memsize */
#define SYS_RCSWC_BF_1024k  (0x07)              /* 1024kb BFW memsize */
#define SYS_RCSWC_MS_128k   (0x10)              /* 128kb ROMCS memsize */
#define SYS_RCSWC_MS_256k   (0x20)              /* 256kb ROMCS memsize */
#define SYS_RCSWC_MS_512k   (0x40)              /* 512kb ROMCS memsize */
#define SYS_RCSWC_MS_1024k  (0x80)              /* 1024kb ROMCS memsize */
#define SYS_RCSWC_MS_MASK   (0xF0)              /* ROMCS memsize mask */
#define SYS_RCSWC_MS_SHIFT  (4)                 /* ROMCS memsize shift */
#define SYS_RCSWC_BFE       (0x08)              /* BFW write enable */
#define SYS_RCSWC_CSS       (0x01)              /* CS swap enable */
#define SYS_RCSWC_HRE       (0x40)              /* half ROM enable */

#define SYS_CSCx_WM         (0x10)

/*
** timing choices may not be relevant to SPI
*/
#define SYS_CSCx_25nSec     (0x00)     /* 40 nSec access time */
#define SYS_CSCx_35nSec     (0x01)     /* 50 nSec access time */
#define SYS_CSCx_50nSec     (0x02)     /* 65 nSec access time */
#define SYS_CSCx_75nSec     (0x03)     /* 90 nSec access time */
#define SYS_CSCx_100nSec    (0x04)     /* 120 nSec access time */
#define SYS_CSCx_150nSec    (0x05)     /* 170 nSec access time */
#define SYS_CSCx_210nSec    (0x06)     /* 280 nSec access time */
#define SYS_CSCx_250nSec    (0x07)     /* 280 nSec access time */


/*****************************************************************************
** SPI Mode Flash Control
*****************************************************************************/

#define SYS_SMMDR   (SYS_CNTL_BASE + 0x080) // SMMDR /* Manual Mode Data Read    */
#define SYS_SMMDW   (SYS_CNTL_BASE + 0x084) // SMMDW /* Manual Mode Data Write   */
#define SYS_SMMC    (SYS_CNTL_BASE + 0x088) // SMMC  /* Manual Mode Control      */
#define SYS_SMMS    (SYS_CNTL_BASE + 0x08C) // SMMS  /* Manual Mode Status       */
#define SYS_SMME    (SYS_CNTL_BASE + 0x090) // SMME  /* Manual Mode Enable       */
//#define SYS_IOM    (SYS_CNTL_BASE + 0x094) // IOM   /* Interface Operating Mode */
#define SYS_FRC    (SYS_CNTL_BASE + 0x09C) // FRC   /* Fast Read Control        */

#define SYS_SMMC_MMSS        0x01     /* Fast Speed Selection*/
#define SYS_SMMC_MMCS(x)     ((x)<<1) /* Manual Mode Chip Select */

#define SYS_SMMS_MMA         0x01  /* Manual Mode Active */
#define SYS_SMMS_NBR         0x02  /* Next Byte Ready */
#define SYS_SMMS_RMMCS       0x80  /* Release Manual Mode Chip Select */

#define SYS_SMME_MME         0x01  /* Manual Mode Enable Mask */

#define SYS_FRC_FRME	    0x01  /* Fast Read Enable */


/*****************************************************************************
** Timers
*****************************************************************************/

#define SYS_TIC0   (SYS_CNTL_BASE + 0x100) /* Timer 0 initial count */
#define SYS_TCC0   (SYS_CNTL_BASE + 0x104) /* Timer 0 current count */
#define SYS_TMC0   (SYS_CNTL_BASE + 0x110) /* Timer 0 mode control */
#define SYS_TIS0   (SYS_CNTL_BASE + 0x114) /* Timer 0 interrupt status */

#define SYS_TIC1   (SYS_CNTL_BASE + 0x180) /* Timer 1 initial count */
#define SYS_TCC1   (SYS_CNTL_BASE + 0x184) /* Timer 1 current count */
#define SYS_TMC1   (SYS_CNTL_BASE + 0x190) /* Timer 1 mode control */
#define SYS_TIS1   (SYS_CNTL_BASE + 0x194) /* Timer 1 interrupt status */

#define SYS_TIC2   (SYS_CNTL_BASE + 0x200) /* Timer 2 initial count */
#define SYS_TCC2   (SYS_CNTL_BASE + 0x204) /* Timer 2 current count */
#define SYS_TMC2   (SYS_CNTL_BASE + 0x200) /* Timer 2 mode control */
#define SYS_TIS2   (SYS_CNTL_BASE + 0x204) /* Timer 2 interrupt status */

#define SYS_TMCx_MASK       (0xFFFFFFFC)            /* Disable timer mask */
#define SYS_TMCx_TE         (0x1)                   /* enable timer */
#define SYS_TMCx_WTM        (0x2)                   /* watch dog timer mode */
#define SYS_TMCx_WT_EN      (SYS_TMCx_TE + SYS_TMCx_WTM) /* enable watch dog timer */

#define SYS_TISx_TIS        (0x2)                   /* timer IRQ status */
#define SYS_TISx_TIE        (0x1)                   /* enable timer interrupt */

/*
** Timer legacy / compatibility stuff
** Define Timer 0 as if it is the Timer in a single timer system
*/
#define SYS_TIC             SYS_TIC0
#define SYS_TCC             SYS_TCC0
#define SYS_TMC             SYS_TMC0
#define SYS_TIS             SYS_TIS0

#define SYS_TMC_TIMEREN     SYS_TMCx_TE
#define SYS_TIS_IRQ         SYS_TISx_TIS
#define SYS_TIS_IRQEN       SYS_TISx_TIE


/*****************************************************************************
** General Purpose UARTS
*****************************************************************************/
#if 1

#define SYS_GPURS0  (SYS_CNTL_BASE + 0x300) /* GP Uart 0 receive status */
#define SYS_GPURD0  (SYS_CNTL_BASE + 0x304) /* GP Uart 0 receive data */
#define SYS_GPUTS0  (SYS_CNTL_BASE + 0x308) /* GP Uart 0 transmit status */
#define SYS_GPUTD0  (SYS_CNTL_BASE + 0x30C) /* GP Uart 0 transmit data */
#define SYS_GPUBR0  (SYS_CNTL_BASE + 0x310) /* GP Uart 0 baud rate select */
#define SYS_GPUIS0  (SYS_CNTL_BASE + 0x318) /* GP Uart 0 Interrupt status */
#define SYS_GPUIE0  (SYS_CNTL_BASE + 0x31C) /* GP Uart 0 Interrupt enable */

#define SYS_GPURS1  (SYS_CNTL_BASE + 0x380) /* GP Uart 1 receive status */
#define SYS_GPURD1  (SYS_CNTL_BASE + 0x384) /* GP Uart 1 receive data */
#define SYS_GPUTS1  (SYS_CNTL_BASE + 0x388) /* GP Uart 1 transmit status */
#define SYS_GPUTD1  (SYS_CNTL_BASE + 0x38C) /* GP Uart 1 transmit data */
#define SYS_GPUBR1  (SYS_CNTL_BASE + 0x390) /* GP Uart 1 baud rate select */
#define SYS_GPUIS1  (SYS_CNTL_BASE + 0x398) /* GP Uart 1 Interrupt status */
#define SYS_GPUIE1  (SYS_CNTL_BASE + 0x39C) /* GP Uart 1 Interrupt enable */

#define SYS_GPUART0_BASE    ( SYS_CNTL_BASE + 0x300 )
#define SYS_GPUART1_BASE    ( SYS_CNTL_BASE + 0x380 )
#else
#define SYS_GPURS0	0x00
#define SYS_GPURD0	0x04
#define SYS_GPUTS0	0x08
#define SYS_GPUTD0	0x0C
#define SYS_GPUBR0	0x10
#define SYS_GPUIS0	0x18
#define SYS_GPUIE0	0x1C

#endif
/*
** UART-related defines
*/
#define SYS_GPURSx_RGPI     (0x80)                  /* Rcv general purpose input */
#define SYS_GPURSx_RXO      (0x02)                  /* Receive Overrun */
#define SYS_GPURSx_RXR      (0x01)                  /* Receive Ready/Int Enable */

#define SYS_GPUTSx_TXS      (0x02)                  /* Transmit Status */
#define SYS_GPUTSx_TXR      (0x01)                  /* Transmit Ready */

#define SYS_GPUBRx_RGPO     (0x80)                  /* Rcv general purpose output */
#define SYS_GPUBRx_TGPO     (0x40)                  /* Tmt general purpose output */
#define SYS_GPUBRx_FSCLRMASK (0xFFFFFFCF)           /* Mask to clear Function Sel */
#define SYS_GPUBRx_FS1      (0x20)                  /* Function Sel -- Int on fall */
#define SYS_GPUBRx_FS0      (0x10)                  /* Function Sel -- Int on rise */
#define SYS_GPUBRx_RFE      (0x08)                  /* Rcv FIFO Enable */
#define SYS_GPUBRx_RATEMASK (0xFFFFFFF8)            /* Mask to clear the BAUD Rate */
#define SYS_GPUBRx_9600     (0x0)
#define SYS_GPUBRx_19200    (0x1)
#define SYS_GPUBRx_38400    (0x2)
#define SYS_GPUBRx_57600    (0x3)
#define SYS_GPUBRx_115200   (0x4)
#define SYS_GPUBRx_230400   (0x5)

#define SYS_GPUISx_GPIS     (0x04)                  /* General purpose int status */
#define SYS_GPUISx_TSIS     (0x02)                  /* Tmt Status Interrupt Status */
#define SYS_GPUISx_TRIS     (0x01)                  /* Tmt Ready Interrupt Status */

#define SYS_GPUIEx_TSIE     (0x02)                  /* Tmt Status Interrupt Enable */
#define SYS_GPUIEx_TRIE     (0x01)                  /* Tmt Ready Interrupt Enable */

/*
** UART legacy / compatibility stuff
*/
#define SYS_SIRS            SYS_GPURS0      // Serial ICE receive status
#define SYS_SIRD            SYS_GPURD0      // Serial ICE receive data
#define SYS_SITS            SYS_GPUTS0      // Serial ICE transmit status
#define SYS_SITD            SYS_GPUTD0      // Serial ICE transmit data
#define SYS_SIBR            SYS_GPUBR0      // Serial ICE baud rate select

#define SYS_SIBR_9600       SYS_GPUBRx_9600
#define SYS_SIBR_19200      SYS_GPUBRx_19200
#define SYS_SIBR_38400      SYS_GPUBRx_38400
#define SYS_SIBR_57600      SYS_GPUBRx_57600
#define SYS_SIBR_115200     SYS_GPUBRx_115200
#define SYS_SIBR_230400     SYS_GPUBRx_230400

#define SYS_SITS_TXR        (0x1)
#define SYS_SIRS_RXR        (0x1)

#define SYS_GPURS           SYS_GPURS1
#define SYS_GPURD           SYS_GPURD1
#define SYS_GPUTS           SYS_GPUTS1
#define SYS_GPUTD           SYS_GPUTD1
#define SYS_GPUBR           SYS_GPUBR1
#define SYS_GPUIS           SYS_GPUIS1
#define SYS_GPUIE           SYS_GPUIE1

#define SYS_GPURS_RGPI      SYS_GPURSx_RGPI
#define SYS_GPURS_RXO       SYS_GPURSx_RXO
#define SYS_GPURS_RXR       SYS_GPURSx_RXR

#define SYS_GPUTS_TXS       SYS_GPUTSx_TXS
#define SYS_GPUTS_TXR       SYS_GPUTSx_TXR

#define SYS_GPUBR_RGPO      SYS_GPUBRx_RGPO
#define SYS_GPUBR_TGPO      SYS_GPUBRx_TGPO
#define SYS_GPUBR_FSCLRMASK SYS_GPUBRx_FSCLRMASK
#define SYS_GPUBR_FS1       SYS_GPUBRx_FS1
#define SYS_GPUBR_FS0       SYS_GPUBRx_FS0
#define SYS_GPUBR_RFE       SYS_GPUBRx_RFE
#define SYS_GPUBR_RATEMASK  SYS_GPUBRx_RATEMASK
#define SYS_GPUBR_9600      SYS_GPUBRx_9600
#define SYS_GPUBR_19200     SYS_GPUBRx_19200
#define SYS_GPUBR_38400     SYS_GPUBRx_38400
#define SYS_GPUBR_57600     SYS_GPUBRx_57600
#define SYS_GPUBR_115200    SYS_GPUBRx_115200
#define SYS_GPUBR_230400    SYS_GPUBRx_230400

#define SYS_GPUIS_GPIS      SYS_GPUISx_GPIS
#define SYS_GPUIS_TSIS      SYS_GPUISx_TSIS
#define SYS_GPUIS_TRIS      SYS_GPUISx_TRIS

#define SYS_GPUIE_TSIE      SYS_GPUIEx_TSIE
#define SYS_GPUIE_TRIE      SYS_GPUIEx_TRIE


/*****************************************************************************
** GTWS subsystem
*****************************************************************************/

#define SYS_GTWS_BASE0      (SYS_CNTL_BASE + 0x500)
#define SYS_GTWS_BASE1      (SYS_CNTL_BASE + 0x580)
#define SYS_GTWS_BASE2      (SYS_CNTL_BASE + 0x600)
#define SYS_GTWS_BASE3      (SYS_CNTL_BASE + 0x680)
#define SYS_GTWS_BASE4      (SYS_CNTL_BASE + 0x700)
#define SYS_GTWS_BASE5      (SYS_CNTL_BASE + 0x780)

#define SYS_SID0            (SYS_GTWS_BASE0 + 0x0)  /* Serial I/F Data */
#define SYS_SIC0            (SYS_GTWS_BASE0 + 0x4)  /* Serial I/F Control */
#define SYS_SIBC0           (SYS_GTWS_BASE0 + 0x8)  /* Serial I/F Bit Ctrl */
#define SYS_SIBS0           (SYS_GTWS_BASE0 + 0xC)  /* Serial I/F Bit Stat */

#define SYS_SID1            (SYS_GTWS_BASE1 + 0x0)  /* Serial I/F Data */
#define SYS_SIC1            (SYS_GTWS_BASE1 + 0x4)  /* Serial I/F Control */
#define SYS_SIBC1           (SYS_GTWS_BASE1 + 0x8)  /* Serial I/F Bit Ctrl */
#define SYS_SIBS1           (SYS_GTWS_BASE1 + 0xC)  /* Serial I/F Bit Stat */

#define SYS_SID2            (SYS_GTWS_BASE2 + 0x0)  /* Serial I/F Data */
#define SYS_SIC2            (SYS_GTWS_BASE2 + 0x4)  /* Serial I/F Control */
#define SYS_SIBC2           (SYS_GTWS_BASE2 + 0x8)  /* Serial I/F Bit Ctrl */
#define SYS_SIBS2           (SYS_GTWS_BASE2 + 0xC)  /* Serial I/F Bit Stat */

#define SYS_SID3            (SYS_GTWS_BASE3 + 0x0)  /* Serial I/F Data */
#define SYS_SIC3            (SYS_GTWS_BASE3 + 0x4)  /* Serial I/F Control */
#define SYS_SIBC3           (SYS_GTWS_BASE3 + 0x8)  /* Serial I/F Bit Ctrl */
#define SYS_SIBS3           (SYS_GTWS_BASE3 + 0xC)  /* Serial I/F Bit Stat */

#define SYS_SID4            (SYS_GTWS_BASE4 + 0x0)  /* Serial I/F Data */
#define SYS_SIC4            (SYS_GTWS_BASE4 + 0x4)  /* Serial I/F Control */
#define SYS_SIBC4           (SYS_GTWS_BASE4 + 0x8)  /* Serial I/F Bit Ctrl */
#define SYS_SIBS4           (SYS_GTWS_BASE4 + 0xC)  /* Serial I/F Bit Stat */

#define SYS_SID5            (SYS_GTWS_BASE5 + 0x0)  /* Serial I/F Data */
#define SYS_SIC5            (SYS_GTWS_BASE5 + 0x4)  /* Serial I/F Control */
#define SYS_SIBC5           (SYS_GTWS_BASE5 + 0x8)  /* Serial I/F Bit Ctrl */
#define SYS_SIBS5           (SYS_GTWS_BASE5 + 0xC)  /* Serial I/F Bit Stat */

#define SYS_GTWS_50KHZ      (0x01)
#define SYS_GTWS_100KHZ     (0x00)
#define SYS_GTWS_400KHZ     (0x04)


/*****************************************************************************
** GPIO Subsystem
*****************************************************************************/

#define SYS_GPD0     (SYS_CNTL_BASE + 0x800) /* GPIO 0 data */
#define SYS_DDP0     (SYS_CNTL_BASE + 0x804) /* GPIO 0 direction */
#define SYS_GPC0     (SYS_CNTL_BASE + 0x808) /* GPIO 0 Port Control */
#define SYS_P0BCIS   (SYS_CNTL_BASE + 0x80C) /* GPIO-0 Bit Ctrl Int St Reg */

#define SYS_GPD1     (SYS_CNTL_BASE + 0x810) /* GPIO 1 data */
#define SYS_DDP1     (SYS_CNTL_BASE + 0x814) /* GPIO 1 direction */
#define SYS_GPC1     (SYS_CNTL_BASE + 0x818) /* GPIO 1 Port Control */
#define SYS_P1BCIS   (SYS_CNTL_BASE + 0x81C) /* GPIO-1 Bit Ctrl Int St Reg */

#define SYS_GPD2     (SYS_CNTL_BASE + 0x820) /* GPIO 2 data */
#define SYS_DDP2     (SYS_CNTL_BASE + 0x824) /* GPIO 2 direction */
#define SYS_GPC2     (SYS_CNTL_BASE + 0x828) /* GPIO 2 Port Control */
#define SYS_P2BCIS   (SYS_CNTL_BASE + 0x82C) /* GPIO-2 Bit Ctrl Int St Reg */

#define SYS_GPD3     (SYS_CNTL_BASE + 0x830) /* GPIO 3 data */
#define SYS_DDP3     (SYS_CNTL_BASE + 0x834) /* GPIO 3 direction */
#define SYS_GPC3     (SYS_CNTL_BASE + 0x838) /* GPIO 3 Port Control */
#define SYS_P3BCIS   (SYS_CNTL_BASE + 0x83C) /* GPIO-3 Bit Ctrl Int St Reg */

#define SYS_GPD4     (SYS_CNTL_BASE + 0x840) /* GPIO 4 data */
#define SYS_DDP4     (SYS_CNTL_BASE + 0x844) /* GPIO 4 direction */
#define SYS_GPC4     (SYS_CNTL_BASE + 0x848) /* GPIO 4 Port Control */
#define SYS_P4BCIS   (SYS_CNTL_BASE + 0x84C) /* GPIO-4 Bit Ctrl Int St Reg */

#define SYS_GPD5     (SYS_CNTL_BASE + 0x850) /* GPIO 5 data */
#define SYS_DDP5     (SYS_CNTL_BASE + 0x854) /* GPIO 5 direction */
#define SYS_GPC5     (SYS_CNTL_BASE + 0x858) /* GPIO 5 Port Control */

/*
** GPIO Masks
*/
#define SYS_GPC4_Bits10_0   0x7FF
#define SYS_PxB0            0x01
#define SYS_PxB1            0x02
#define SYS_PxB2            0x04
#define SYS_PxB3            0x08
#define SYS_PxB4            0x10
#define SYS_PxB5            0x20
#define SYS_PxB6            0x40
#define SYS_PxB7            0x80
#define SYS_PxB8            0x100
#define SYS_PxB9            0x200
#define SYS_PxB10           0x400
#define SYS_PxB11           0x800
#define SYS_PxB12           0x1000
#define SYS_PxB13           0x2000
#define SYS_PxB14           0x4000
#define SYS_PxB15           0x8000
#define SYS_PxB16           0x10000
#define SYS_PxB17           0x20000
#define SYS_PxB18           0x40000
#define SYS_PxB19           0x80000
#define SYS_PxB20           0x100000
#define SYS_PxB21           0x200000
#define SYS_PxB22           0x400000
#define SYS_PxB23           0x800000

/*****************************************************************************
** Option Control Register
*****************************************************************************/

#define SYS_OCR             (SYS_CNTL_BASE + 0x8A4) /* Option Control Register */

#define SYS_OCR_ROL_P1B2    0x01        /* RTS Output Level for P1_2 */
#define SYS_OCR_RFL_P1B2    0x02        /* RTS FIFO Level for P1_2 */
#define SYS_OCR_ROE_P1B2    0x04        /* RTS Output Enable for P1_2 */
#define SYS_OCR_AUE         0x08        /* Auxillary UART Enable */
#define SYS_OCR_ROL_P1B3    0x10        /* RTS Output Level for P1_3 */
#define SYS_OCR_RFL_P1B3    0x20        /* RTS FIFO Level for P1_3 */
#define SYS_OCR_ROE_P1B3    0x40        /* RTS Output Enable for P1_3 */
#define SYS_OCR_SIE         0x80        /* SFF8485 Interface Enable */
#define SYS_OCS_P0_22       0xF00       /* OCS on P0_22 */
#define SYS_OCS_P0_23       0xF000      /* OCS on P0_23 */

#define SYS_OCR_DIS_P0B22       0x0000      /* Disabled */
#define SYS_OCR_37_5MHZ_P0B22   0x0100      /* Divide by 2, 37.5MHz output (50% duty cycle) */
#define SYS_OCR_25_0MHZ_P0B22   0x0200      /* Divide by 3, 25.0MHz output (67% duty cycle) */
#define SYS_OCR_18_75MHZ_P0B22  0x0300      /* Divide by 4, 18.75MHz output (50% duty cycle) */
#define SYS_OCR_15_0MHZ_P0B22   0x0400      /* Divide by 5, 15.0MHz output (60% duty cycle) */
#define SYS_OCR_12_5MHZ_P0B22   0x0500      /* Divide by 6, 12.5MHz output (67% duty cycle */
#define SYS_OCR_10_71MHZ_P0B22  0x0600      /* Divide by 7, 10.71MHz output (57% duty cycle) */
#define SYS_OCR_9_37MHZ_P0B22   0x0700      /* Divide by 8, 9.37MHz output (50% duty cycle) */
#define SYS_OCR_8_33MHZ_P0B22   0x0800      /* Divide by 9, 8.33MHz output (55% duty cycle) VSC055 compatible */
#define SYS_OCR_7_5MHZ_P0B22    0x0900      /* Divide by 10, 7.5MHz output (60% duty cycle) */
#define SYS_OCR_6_82MHZ_P0B22   0x0A00      /* Divide by 11, 6.82MHz output (64% duty cycle) */
#define SYS_OCR_6_25MHZ_P0B22   0x0B00      /* Divide by 12, 6.25MHz output (67% duty cycle) */
#define SYS_OCR_5_77MHZ_P0B22   0x0C00      /* Divide by 13, 5.77MHz output (62% duty cycle) */
#define SYS_OCR_5_38MHZ_P0B22   0x0D00      /* Divide by 14, 5.38MHz output (57% duty cycle) */
#define SYS_OCR_5_0MHZ_P0B22    0x0E00      /* Divide by 15, 5.0MHz output (53% duty cycle) */
#define SYS_OCR_4_68MHZ_P0B22   0x0F00      /* Divide by 16, 4.68MHz output (50% duty cycle) */

#define SYS_OCR_DIS_P0B23       0x00000     /* Disabled */
#define SYS_OCR_37_5MHZ_P0B23   0x01000     /* Divide by 2, 37.5MHz output (50% duty cycle) */
#define SYS_OCR_25_0MHZ_P0B23   0x02000     /* Divide by 3, 25.0MHz output (67% duty cycle) */
#define SYS_OCR_18_75MHZ_P0B23  0x03000     /* Divide by 4, 18.75MHz output (50% duty cycle) */
#define SYS_OCR_15_0MHZ_P0B23   0x04000     /* Divide by 5, 15.0MHz output (60% duty cycle) */
#define SYS_OCR_12_5MHZ_P0B23   0x05000     /* Divide by 6, 12.5MHz output (67% duty cycle */
#define SYS_OCR_10_71MHZ_P0B23  0x06000     /* Divide by 7, 10.71MHz output (57% duty cycle) */
#define SYS_OCR_9_37MHZ_P0B23   0x07000     /* Divide by 8, 9.37MHz output (50% duty cycle) */
#define SYS_OCR_8_33MHZ_P0B23   0x08000     /* Divide by 9, 8.33MHz output (55% duty cycle) VSC055 compatible */
#define SYS_OCR_7_5MHZ_P0B23    0x09000     /* Divide by 10, 7.5MHz output (60% duty cycle) */
#define SYS_OCR_6_82MHZ_P0B23   0x0A000     /* Divide by 11, 6.82MHz output (64% duty cycle) */
#define SYS_OCR_6_25MHZ_P0B23   0x0B000     /* Divide by 12, 6.25MHz output (67% duty cycle) */
#define SYS_OCR_5_77MHZ_P0B23   0x0C000     /* Divide by 13, 5.77MHz output (62% duty cycle) */
#define SYS_OCR_5_38MHZ_P0B23   0x0D000     /* Divide by 14, 5.38MHz output (57% duty cycle) */
#define SYS_OCR_5_0MHZ_P0B23    0x0E000     /* Divide by 15, 5.0MHz output (53% duty cycle) */
#define SYS_OCR_4_68MHZ_P0B23   0x0F000     /* Divide by 16, 4.68MHz output (50% duty cycle) */

/*****************************************************************************
** Fan Control
*****************************************************************************/

#define SYS_FSC        (SYS_CNTL_BASE + 0x8AC) /* Fan Speed Control */
#define SYS_FSCO       (SYS_CNTL_BASE + 0x8B0) /* Fan Speed Count Overflow */
#define SYS_FSCU       (SYS_CNTL_BASE + 0x8B4) /* Fan Speed Count Underflow */
#define SYS_FSCC       (SYS_CNTL_BASE + 0x8B8) /* Fan Speed Current Count */
#define SYS_FSIS       (SYS_CNTL_BASE + 0x8BC) /* Fan Speed Interrupt Status */

#define FAN_TACH_0          0x00        /* Tach Input Port 1, bit 4 */
#define FAN_TACH_1          0x01        /* Tach Input Port 1, bit 5 */
#define FAN_TACH_2          0x02        /* Tach Input Port 1, bit 6 */
#define FAN_TACH_3          0x03        /* Tach Input Port 1, bit 7 */
#define FAN_TACH_4          0x04        /* Tach Input Port 0, bit 22 */
#define FAN_TACH_5          0x05        /* Tach Input Port 0, bit 23 */

#define SYS_FSC_FD1         0x00        /* Fan Speed Divisor 1, RPM 8000, Clock 19.2KHz */
#define SYS_FSC_FD2         0x08        /* Fan Speed Divisor 2, RPM 4000, Clock 9.6KHz */
#define SYS_FSC_FD4         0x10        /* Fan Speed Divisor 4, RPM 2000, Clock 4.8KHz */
#define SYS_FSC_FD8         0x18        /* Fan Speed Divisor 8, RPM 1000, Clock 2.4KHz */

#define FAN_IRQ_TOIE        (0x40)
#define FAN_IRQ_TUIE        (0x20)
#define FAN_IRQ_BOTH        (0x60)
#define FAN_IRQ_DISABLE     (0x00)
#define FAN_CLEAR_IRQ       (0x01)

#define FAN_DISABLE_TACH    (0x00)
#define FAN_ENABLE_TACH     (0x80)

#define SYS_FSC_FSE         0x80        /* Fan Speed Monitoring Mode Enable */


#define SYS_GPD        0x1
#define SYS_DD         0x2
#define SYS_FS         0xC
#define SYS_LPS        0xF0


/*****************************************************************************
** Pulse Width Modulation
*****************************************************************************/

#define SYS_PWMC0    (SYS_CNTL_BASE + 0x8C0)     /* PWM Control 0 Register */
#define SYS_PWMC1    (SYS_CNTL_BASE + 0x8C4)     /* PWM Control 1 Register */
#define SYS_PWMC2    (SYS_CNTL_BASE + 0x8C8)     /* PWM Control 2 Register */
#define SYS_PWMC3    (SYS_CNTL_BASE + 0x8CC)     /* PWM Control 3 Register */
#define SYS_PWMC4    (SYS_CNTL_BASE + 0x8D0)     /* PWM Control 4 Register */
#define SYS_PWMC6    (SYS_CNTL_BASE + 0x8D4)     /* PWM Control 6 Register */
#define SYS_PWMC5    (SYS_CNTL_BASE + 0x8D8)     /* PWM Control 5 Register */
#define SYS_PWMC7    (SYS_CNTL_BASE + 0x8DC)     /* PWM Control 7 Register */

#define SYS_PWMD     (SYS_CNTL_BASE + 0x8E0)     /* PWM Divider */
#define SYS_PWM_PTC0 (SYS_CNTL_BASE + 0x8E4)     /* PWM pulse train ctrl 0 */
#define SYS_PWM_PTC1 (SYS_CNTL_BASE + 0x8E8)     /* PWM pulse train ctrl 1 */
#define SYS_PWM_PTC2 (SYS_CNTL_BASE + 0x8EC)     /* PWM pulse train ctrl 2 */


/*****************************************************************************
** special CPU and/or chip registers
*****************************************************************************/

#define SYS_CTR    (SYS_CNTL_BASE + 0x08F0) /* chip type and revision */
#define SYS_HIF    (SYS_CNTL_BASE + 0x08F4) /* hardware interrupt forcing */
#define SYS_IOR    (SYS_CNTL_BASE + 0x08F8) /* interrupt observe */


/*****************************************************************************
** GPIO Port Bit Controls
*****************************************************************************/

#define SYS_GP0BC0     (SYS_CNTL_BASE + 0xA00) /* GPIO Port 0 Bit Ctrl 0 Reg */
#define SYS_GP0BC1     (SYS_CNTL_BASE + 0xA04) /* GPIO Port 0 Bit Ctrl 1 Reg */
#define SYS_GP0BC2     (SYS_CNTL_BASE + 0xA08) /* GPIO Port 0 Bit Ctrl 2 Reg */
#define SYS_GP0BC3     (SYS_CNTL_BASE + 0xA0C) /* GPIO Port 0 Bit Ctrl 3 Reg */
#define SYS_GP0BC4     (SYS_CNTL_BASE + 0xA10) /* GPIO Port 0 Bit Ctrl 4 Reg */
#define SYS_GP0BC5     (SYS_CNTL_BASE + 0xA14) /* GPIO Port 0 Bit Ctrl 5 Reg */
#define SYS_GP0BC6     (SYS_CNTL_BASE + 0xA18) /* GPIO Port 0 Bit Ctrl 6 Reg */
#define SYS_GP0BC7     (SYS_CNTL_BASE + 0xA1C) /* GPIO Port 0 Bit Ctrl 7 Reg */
#define SYS_GP0BC8     (SYS_CNTL_BASE + 0xA20) /* GPIO Port 0 Bit Ctrl 8 Reg */
#define SYS_GP0BC9     (SYS_CNTL_BASE + 0xA24) /* GPIO Port 0 Bit Ctrl 9 Reg */
#define SYS_GP0BC10    (SYS_CNTL_BASE + 0xA28) /* GPIO Port 0 Bit Ctrl 10 Reg */
#define SYS_GP0BC11    (SYS_CNTL_BASE + 0xA2C) /* GPIO Port 0 Bit Ctrl 11 Reg */
#define SYS_GP0BC12    (SYS_CNTL_BASE + 0xA30) /* GPIO Port 0 Bit Ctrl 12 Reg */
#define SYS_GP0BC13    (SYS_CNTL_BASE + 0xA34) /* GPIO Port 0 Bit Ctrl 13 Reg */
#define SYS_GP0BC14    (SYS_CNTL_BASE + 0xA38) /* GPIO Port 0 Bit Ctrl 14 Reg */
#define SYS_GP0BC15    (SYS_CNTL_BASE + 0xA3C) /* GPIO Port 0 Bit Ctrl 15 Reg */

#define SYS_GP1BC0     (SYS_CNTL_BASE + 0xA40) /* GPIO Port 1 Bit Ctrl 0 Reg */
#define SYS_GP1BC1     (SYS_CNTL_BASE + 0xA44) /* GPIO Port 1 Bit Ctrl 1 Reg */
#define SYS_GP1BC2     (SYS_CNTL_BASE + 0xA48) /* GPIO Port 1 Bit Ctrl 2 Reg */
#define SYS_GP1BC3     (SYS_CNTL_BASE + 0xA4C) /* GPIO Port 1 Bit Ctrl 3 Reg */
#define SYS_GP1BC4     (SYS_CNTL_BASE + 0xA50) /* GPIO Port 1 Bit Ctrl 4 Reg */
#define SYS_GP1BC5     (SYS_CNTL_BASE + 0xA54) /* GPIO Port 1 Bit Ctrl 5 Reg */
#define SYS_GP1BC6     (SYS_CNTL_BASE + 0xA58) /* GPIO Port 1 Bit Ctrl 6 Reg */
#define SYS_GP1BC7     (SYS_CNTL_BASE + 0xA5C) /* GPIO Port 1 Bit Ctrl 7 Reg */
#define SYS_GP1BC8     (SYS_CNTL_BASE + 0xA60) /* GPIO Port 1 Bit Ctrl 8 Reg */
#define SYS_GP1BC9     (SYS_CNTL_BASE + 0xA64) /* GPIO Port 1 Bit Ctrl 9 Reg */
#define SYS_GP1BC10    (SYS_CNTL_BASE + 0xA68) /* GPIO Port 1 Bit Ctrl 10 Reg */
#define SYS_GP1BC11    (SYS_CNTL_BASE + 0xA6C) /* GPIO Port 1 Bit Ctrl 11 Reg */
#define SYS_GP1BC12    (SYS_CNTL_BASE + 0xA70) /* GPIO Port 1 Bit Ctrl 12 Reg */
#define SYS_GP1BC13    (SYS_CNTL_BASE + 0xA74) /* GPIO Port 1 Bit Ctrl 13 Reg */
#define SYS_GP1BC14    (SYS_CNTL_BASE + 0xA78) /* GPIO Port 1 Bit Ctrl 14 Reg */
#define SYS_GP1BC15    (SYS_CNTL_BASE + 0xA7C) /* GPIO Port 1 Bit Ctrl 15 Reg */

#define SYS_GP2BC0     (SYS_CNTL_BASE + 0xA80) /* GPIO Port 2 Bit Ctrl 0 Reg */
#define SYS_GP2BC1     (SYS_CNTL_BASE + 0xA84) /* GPIO Port 2 Bit Ctrl 1 Reg */
#define SYS_GP2BC2     (SYS_CNTL_BASE + 0xA88) /* GPIO Port 2 Bit Ctrl 2 Reg */
#define SYS_GP2BC3     (SYS_CNTL_BASE + 0xA8C) /* GPIO Port 2 Bit Ctrl 3 Reg */
#define SYS_GP2BC4     (SYS_CNTL_BASE + 0xA90) /* GPIO Port 2 Bit Ctrl 4 Reg */
#define SYS_GP2BC5     (SYS_CNTL_BASE + 0xA94) /* GPIO Port 2 Bit Ctrl 5 Reg */
#define SYS_GP2BC6     (SYS_CNTL_BASE + 0xA98) /* GPIO Port 2 Bit Ctrl 6 Reg */
#define SYS_GP2BC7     (SYS_CNTL_BASE + 0xA9C) /* GPIO Port 2 Bit Ctrl 7 Reg */
#define SYS_GP2BC8     (SYS_CNTL_BASE + 0xAA0) /* GPIO Port 2 Bit Ctrl 8 Reg */
#define SYS_GP2BC9     (SYS_CNTL_BASE + 0xAA4) /* GPIO Port 2 Bit Ctrl 9 Reg */
#define SYS_GP2BC10    (SYS_CNTL_BASE + 0xAA8) /* GPIO Port 2 Bit Ctrl 10 Reg */
#define SYS_GP2BC11    (SYS_CNTL_BASE + 0xAAC) /* GPIO Port 2 Bit Ctrl 11 Reg */
#define SYS_GP2BC12    (SYS_CNTL_BASE + 0xAB0) /* GPIO Port 2 Bit Ctrl 12 Reg */
#define SYS_GP2BC13    (SYS_CNTL_BASE + 0xAB4) /* GPIO Port 2 Bit Ctrl 13 Reg */
#define SYS_GP2BC14    (SYS_CNTL_BASE + 0xAB8) /* GPIO Port 2 Bit Ctrl 14 Reg */
#define SYS_GP2BC15    (SYS_CNTL_BASE + 0xABC) /* GPIO Port 2 Bit Ctrl 15 Reg */

#define SYS_GP3BC0     (SYS_CNTL_BASE + 0xAC0) /* GPIO Port 3 Bit Ctrl 0 Reg */
#define SYS_GP3BC1     (SYS_CNTL_BASE + 0xAC4) /* GPIO Port 3 Bit Ctrl 1 Reg */
#define SYS_GP3BC2     (SYS_CNTL_BASE + 0xAC8) /* GPIO Port 3 Bit Ctrl 2 Reg */
#define SYS_GP3BC3     (SYS_CNTL_BASE + 0xACC) /* GPIO Port 3 Bit Ctrl 3 Reg */
#define SYS_GP3BC4     (SYS_CNTL_BASE + 0xAD0) /* GPIO Port 3 Bit Ctrl 4 Reg */
#define SYS_GP3BC5     (SYS_CNTL_BASE + 0xAD4) /* GPIO Port 3 Bit Ctrl 5 Reg */
#define SYS_GP3BC6     (SYS_CNTL_BASE + 0xAD8) /* GPIO Port 3 Bit Ctrl 6 Reg */
#define SYS_GP3BC7     (SYS_CNTL_BASE + 0xADC) /* GPIO Port 3 Bit Ctrl 7 Reg */
#define SYS_GP3BC8     (SYS_CNTL_BASE + 0xAE0) /* GPIO Port 3 Bit Ctrl 8 Reg */
#define SYS_GP3BC9     (SYS_CNTL_BASE + 0xAE4) /* GPIO Port 3 Bit Ctrl 9 Reg */
#define SYS_GP3BC10    (SYS_CNTL_BASE + 0xAE8) /* GPIO Port 3 Bit Ctrl 10 Reg */
#define SYS_GP3BC11    (SYS_CNTL_BASE + 0xAEC) /* GPIO Port 3 Bit Ctrl 11 Reg */
#define SYS_GP3BC12    (SYS_CNTL_BASE + 0xAF0) /* GPIO Port 3 Bit Ctrl 12 Reg */
#define SYS_GP3BC13    (SYS_CNTL_BASE + 0xAF4) /* GPIO Port 3 Bit Ctrl 13 Reg */
#define SYS_GP3BC14    (SYS_CNTL_BASE + 0xAF8) /* GPIO Port 3 Bit Ctrl 14 Reg */
#define SYS_GP3BC15    (SYS_CNTL_BASE + 0xAFC) /* GPIO Port 3 Bit Ctrl 15 Reg */

#define SYS_GP4BC0     (SYS_CNTL_BASE + 0xB00) /* GPIO Port 4 Bit Ctrl 0 Reg */
#define SYS_GP4BC1     (SYS_CNTL_BASE + 0xB04) /* GPIO Port 4 Bit Ctrl 1 Reg */
#define SYS_GP4BC2     (SYS_CNTL_BASE + 0xB08) /* GPIO Port 4 Bit Ctrl 2 Reg */
#define SYS_GP4BC3     (SYS_CNTL_BASE + 0xB0C) /* GPIO Port 4 Bit Ctrl 3 Reg */
#define SYS_GP4BC4     (SYS_CNTL_BASE + 0xB10) /* GPIO Port 4 Bit Ctrl 4 Reg */
#define SYS_GP4BC5     (SYS_CNTL_BASE + 0xB14) /* GPIO Port 4 Bit Ctrl 5 Reg */
#define SYS_GP4BC6     (SYS_CNTL_BASE + 0xB18) /* GPIO Port 4 Bit Ctrl 6 Reg */
#define SYS_GP4BC7     (SYS_CNTL_BASE + 0xB1C) /* GPIO Port 4 Bit Ctrl 7 Reg */
#define SYS_GP4BC8     (SYS_CNTL_BASE + 0xB20) /* GPIO Port 4 Bit Ctrl 8 Reg */
#define SYS_GP4BC9     (SYS_CNTL_BASE + 0xB24) /* GPIO Port 4 Bit Ctrl 9 Reg */
#define SYS_GP4BC10    (SYS_CNTL_BASE + 0xB28) /* GPIO Port 4 Bit Ctrl 10 Reg */
#define SYS_GP4BC11    (SYS_CNTL_BASE + 0xB2C) /* GPIO Port 4 Bit Ctrl 11 Reg */
#define SYS_GP4BC12    (SYS_CNTL_BASE + 0xB30) /* GPIO Port 4 Bit Ctrl 12 Reg */
#define SYS_GP4BC13    (SYS_CNTL_BASE + 0xB34) /* GPIO Port 4 Bit Ctrl 13 Reg */
#define SYS_GP4BC14    (SYS_CNTL_BASE + 0xB38) /* GPIO Port 4 Bit Ctrl 14 Reg */
#define SYS_GP4BC15    (SYS_CNTL_BASE + 0xB3C) /* GPIO Port 4 Bit Ctrl 15 Reg */

#define SYS_GP5BC0     (SYS_CNTL_BASE + 0xB40) /* GPIO Port 5 Bit Ctrl 0 Reg */
#define SYS_GP5BC1     (SYS_CNTL_BASE + 0xB44) /* GPIO Port 5 Bit Ctrl 1 Reg */
#define SYS_GP5BC2     (SYS_CNTL_BASE + 0xB48) /* GPIO Port 5 Bit Ctrl 2 Reg */
#define SYS_GP5BC3     (SYS_CNTL_BASE + 0xB4C) /* GPIO Port 5 Bit Ctrl 3 Reg */
#define SYS_GP5BC4     (SYS_CNTL_BASE + 0xB50) /* GPIO Port 5 Bit Ctrl 4 Reg */
#define SYS_GP5BC5     (SYS_CNTL_BASE + 0xB54) /* GPIO Port 5 Bit Ctrl 5 Reg */
#define SYS_GP5BC6     (SYS_CNTL_BASE + 0xB58) /* GPIO Port 5 Bit Ctrl 6 Reg */
#define SYS_GP5BC7     (SYS_CNTL_BASE + 0xB5C) /* GPIO Port 5 Bit Ctrl 7 Reg */
#define SYS_GP5BC8     (SYS_CNTL_BASE + 0xB60) /* GPIO Port 5 Bit Ctrl 8 Reg */
#define SYS_GP5BC9     (SYS_CNTL_BASE + 0xB64) /* GPIO Port 5 Bit Ctrl 9 Reg */
#define SYS_GP5BC10    (SYS_CNTL_BASE + 0xB68) /* GPIO Port 5 Bit Ctrl 10 Reg */
#define SYS_GP5BC11    (SYS_CNTL_BASE + 0xB6C) /* GPIO Port 5 Bit Ctrl 11 Reg */
#define SYS_GP5BC12    (SYS_CNTL_BASE + 0xB70) /* GPIO Port 5 Bit Ctrl 12 Reg */
#define SYS_GP5BC13    (SYS_CNTL_BASE + 0xB74) /* GPIO Port 5 Bit Ctrl 13 Reg */
#define SYS_GP5BC14    (SYS_CNTL_BASE + 0xB78) /* GPIO Port 5 Bit Ctrl 14 Reg */
#define SYS_GP5BC15    (SYS_CNTL_BASE + 0xB7C) /* GPIO Port 5 Bit Ctrl 15 Reg */
#define SYS_GP5BC16    (SYS_CNTL_BASE + 0xB80) /* GPIO Port 5 Bit Ctrl 16 Reg */
#define SYS_GP5BC17    (SYS_CNTL_BASE + 0xB84) /* GPIO Port 5 Bit Ctrl 17 Reg */
#define SYS_GP5BC18    (SYS_CNTL_BASE + 0xB88) /* GPIO Port 5 Bit Ctrl 18 Reg */
#define SYS_GP5BC19    (SYS_CNTL_BASE + 0xB8C) /* GPIO Port 5 Bit Ctrl 19 Reg */
#define SYS_GP5BC20    (SYS_CNTL_BASE + 0xB90) /* GPIO Port 5 Bit Ctrl 20 Reg */
#define SYS_GP5BC21    (SYS_CNTL_BASE + 0xB94) /* GPIO Port 5 Bit Ctrl 21 Reg */
#define SYS_GP5BC22    (SYS_CNTL_BASE + 0xB98) /* GPIO Port 5 Bit Ctrl 22 Reg */
#define SYS_GP5BC23    (SYS_CNTL_BASE + 0xB9C) /* GPIO Port 5 Bit Ctrl 23 Reg */


#endif  /* end of ifndef __VSC452_H_ */
